@@ -12,13 +12,16 @@
 ##############################################################################
 """Access control support
 
-$Id: Role.py 73389 2007-03-20 09:03:57Z mj $
+$Id: Role.py 77297 2007-07-02 10:14:18Z shh $
 """
 from cgi import escape
 
 from Globals import DTMLFile, MessageDialog, Dictionary
 from Acquisition import Implicit, Acquired, aq_get
-import Globals, ExtensionClass, PermissionMapping, Products
+from Globals import InitializeClass
+from AccessControl import ClassSecurityInfo
+from AccessControl.Permissions import change_permissions
+import ExtensionClass, PermissionMapping, Products
 from App.Common import aq_base
 from zope.interface import implements
 
@@ -42,21 +45,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
 
     implements(IRoleManager)
 
-    __ac_permissions__=(
-        ('Change permissions',
-         ('manage_access', 'permission_settings',
-          'ac_inherited_permissions',
-          'manage_roleForm', 'manage_role',
-          'manage_acquiredForm', 'manage_acquiredPermissions',
-          'manage_permissionForm', 'manage_permission',
-          'manage_changePermissions', 'permissionsOfRole',
-          'rolesOfPermission', 'acquiredRolesAreUsedBy',
-          'manage_defined_roles', 'userdefined_roles',
-          'manage_listLocalRoles', 'manage_editLocalRoles',
-          'manage_setLocalRoles', 'manage_addLocalRoles',
-          'manage_delLocalRoles'
-          )),
-        )
+    security = ClassSecurityInfo()
 
     manage_options=(
         {'label':'Security', 'action':'manage_access',
@@ -75,6 +64,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
 
     #------------------------------------------------------------
 
+    security.declareProtected(change_permissions, 'ac_inherited_permissions')
     def ac_inherited_permissions(self, all=0):
         # Get all permissions not defined in ourself that are inherited
         # This will be a sequence of tuples with a name as the first item and
@@ -97,6 +87,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
 
         return tuple(r)
 
+    security.declareProtected(change_permissions, 'permission_settings')
     def permission_settings(self, permission=None):
         """Return user-role permission settings.
 
@@ -131,11 +122,13 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
             result.append(d)
         return result
 
+    security.declareProtected(change_permissions, 'manage_roleForm')
     manage_roleForm=DTMLFile('dtml/roleEdit', globals(),
                              management_view='Security',
                              help_topic='Security_Manage-Role.stx',
                              help_product='OFSP')
 
+    security.declareProtected(change_permissions, 'manage_role')
     @postonly
     def manage_role(self, role_to_manage, permissions=[], REQUEST=None):
         """Change the permissions given to the given role.
@@ -148,11 +141,13 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
 
         if REQUEST is not None: return self.manage_access(REQUEST)
 
+    security.declareProtected(change_permissions, 'manage_acquiredForm')
     manage_acquiredForm=DTMLFile('dtml/acquiredEdit', globals(),
                                  management_view='Security',
                                  help_topic='Security_Manage-Acquisition.stx',
                                  help_product='OFSP')
 
+    security.declareProtected(change_permissions, 'manage_acquiredPermissions')
     @postonly
     def manage_acquiredPermissions(self, permissions=[], REQUEST=None):
         """Change the permissions that acquire.
@@ -168,11 +163,13 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
 
         if REQUEST is not None: return self.manage_access(REQUEST)
 
+    security.declareProtected(change_permissions, 'manage_permissionForm')
     manage_permissionForm=DTMLFile('dtml/permissionEdit', globals(),
                                    management_view='Security',
                                    help_topic='Security_Manage-Permission.stx',
                                    help_product='OFSP')
 
+    security.declareProtected(change_permissions, 'manage_permission')
     @postonly
     def manage_permission(self, permission_to_manage,
                           roles=[], acquire=0, REQUEST=None):
@@ -201,6 +198,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
 
     _method_manage_access=DTMLFile('dtml/methodAccess', globals())
 
+    security.declareProtected(change_permissions, 'manage_access')
     def manage_access(self, REQUEST, **kw):
         """Return an interface for making permissions settings.
         """
@@ -210,6 +208,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
         else:
             return apply(self._normal_manage_access,(), kw)
 
+    security.declareProtected(change_permissions, 'manage_changePermissions')
     @postonly
     def manage_changePermissions(self, REQUEST):
         """Change all permissions settings, called by management screen.
@@ -242,6 +241,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
             message='Your changes have been saved',
             action ='manage_access')
 
+    security.declareProtected(change_permissions, 'permissionsOfRole')
     def permissionsOfRole(self, role):
         """Used by management screen.
         """
@@ -255,6 +255,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
                       })
         return r
 
+    security.declareProtected(change_permissions, 'rolesOfPermission')
     def rolesOfPermission(self, permission):
         """Used by management screen.
         """
@@ -274,6 +275,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
         raise ValueError, (
             "The permission <em>%s</em> is invalid." % escape(permission))
 
+    security.declareProtected(change_permissions, 'acquiredRolesAreUsedBy')
     def acquiredRolesAreUsedBy(self, permission):
         """Used by management screen.
         """
@@ -298,11 +300,13 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
 
     __ac_local_roles__=None
 
+    security.declareProtected(change_permissions, 'manage_listLocalRoles')
     manage_listLocalRoles=DTMLFile('dtml/listLocalRoles', globals(),
                                    management_view='Security',
                                    help_topic='Security_Local-Roles.stx',
                                    help_product='OFSP')
 
+    security.declareProtected(change_permissions, 'manage_editLocalRoles')
     manage_editLocalRoles=DTMLFile('dtml/editLocalRoles', globals(),
                                    management_view='Security',
                                    help_topic='Security_User-Local-Roles.stx',
@@ -341,11 +345,12 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
                 if mlu < 0: raise OverflowError
                 un = getattr(aclu, 'user_names', _notfound)
                 if un is not _notfound:
+                    un = aclu.__of__(item).user_names # rewrap
                     unl = un()
                     # maxlistusers of 0 is list all
                     if len(unl) > mlu and mlu != 0:
                         raise OverflowError
-                    for name in un():
+                    for name in unl:
                         dict[name]=1
             item = getattr(item, 'aq_parent', _notfound)
             if item is _notfound:
@@ -358,6 +363,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
         dict=self.__ac_local_roles__ or {}
         return tuple(dict.get(userid, []))
 
+    security.declareProtected(change_permissions, 'manage_addLocalRoles')
     @postonly
     def manage_addLocalRoles(self, userid, roles, REQUEST=None):
         """Set local roles for a user."""
@@ -376,6 +382,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
             stat='Your changes have been saved.'
             return self.manage_listLocalRoles(self, REQUEST, stat=stat)
 
+    security.declareProtected(change_permissions, 'manage_setLocalRoles')
     @postonly
     def manage_setLocalRoles(self, userid, roles, REQUEST=None):
         """Set local roles for a user."""
@@ -390,6 +397,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
             stat='Your changes have been saved.'
             return self.manage_listLocalRoles(self, REQUEST, stat=stat)
 
+    security.declareProtected(change_permissions, 'manage_delLocalRoles')
     @postonly
     def manage_delLocalRoles(self, userids, REQUEST=None):
         """Remove all local roles for a user."""
@@ -406,7 +414,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
 
     #------------------------------------------------------------
 
-    access_debug_info__roles__=()
+    security.declarePrivate('access_debug_info')
     def access_debug_info(self):
         """Return debug info.
         """
@@ -458,6 +466,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
                 return 0
         return 1
 
+    security.declareProtected(change_permissions, 'userdefined_roles')
     def userdefined_roles(self):
         """Return list of user-defined roles.
         """
@@ -467,12 +476,13 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
             except: pass
         return tuple(roles)
 
+    security.declareProtected(change_permissions, 'manage_defined_roles')
     def manage_defined_roles(self, submit=None, REQUEST=None):
         """Called by management screen.
         """
 
         if submit=='Add Role':
-            role=reqattr(REQUEST, 'role')
+            role=reqattr(REQUEST, 'role').strip()
             return self._addRole(role, REQUEST)
 
         if submit=='Delete Role':
@@ -544,7 +554,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
 
         return d
 
-Globals.default__class_init__(RoleManager)
+InitializeClass(RoleManager)
 
 
 def reqattr(request, attr):
