@@ -4,7 +4,7 @@
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -12,8 +12,8 @@
 #
 ##############################################################################
 
-import OOBTree, Interface
-from Interface import Interface
+from zope.interface import Interface
+
 
 class ICollection(Interface):
 
@@ -48,26 +48,33 @@ class IKeyed(ICollection):
     def has_key(key):
         """Check whether the object has an item with the given key.
 
-        Return a true value if so, else a false value.
+        Return a true value if the key is present, else a false value.
         """
 
-    def keys(min=None, max=None):
+    def keys(min=None, max=None, excludemin=False, excludemax=False):
         """Return an IReadSequence containing the keys in the collection.
 
-        The type of the IReadSequence is not specified.  It could be a
-        list or a tuple or some other type.
+        The type of the IReadSequence is not specified.  It could be a list
+        or a tuple or some other type.
 
-        If a min is specified, then output is constrained to
-        items having keys greater than or equal to the given min.
-        A min value of None is ignored.
+        All arguments are optional, and may be specified as keyword
+        arguments, or by position.
 
-        If a max is specified, then output is constrained to
-        items having keys less than or equal to the given min.
-        A max value of None is ignored.
+        If a min is specified, then output is constrained to keys greater
+        than or equal to the given min, and, if excludemin is specified and
+        true, is further constrained to keys strictly greater than min.  A
+        min value of None is ignored.  If min is None or not specified, and
+        excludemin is true, the smallest key is excluded.
+
+        If a max is specified, then output is constrained to keys less than
+        or equal to the given max, and, if excludemax is specified and
+        true, is further constrained to keys strictly less than max.  A max
+        value of None is ignored.  If max is None or not specified, and
+        excludemax is true, the largest key is excluded.
         """
 
     def maxKey(key=None):
-        """Return the maximum key
+        """Return the maximum key.
 
         If a key argument if provided and not None, return the largest key
         that is less than or equal to the argument.  Raise an exception if
@@ -75,13 +82,14 @@ class IKeyed(ICollection):
         """
 
     def minKey(key=None):
-        """Return the minimum key
+        """Return the minimum key.
 
         If a key argument if provided and not None, return the smallest key
         that is greater than or equal to the argument.  Raise an exception
         if no such key exists.
         """
 
+
 class ISetMutable(IKeyed):
 
     def insert(key):
@@ -96,12 +104,14 @@ class ISetMutable(IKeyed):
     def update(seq):
         """Add the items from the given sequence to the set."""
 
+
 class ISized(Interface):
-    "anything supporting __len"
+    """An object that supports __len__."""
 
     def __len__():
         """Return the number of items in the container."""
 
+
 class IKeySequence(IKeyed, ISized):
 
     def __getitem__(index):
@@ -111,20 +121,15 @@ class IKeySequence(IKeyed, ISized):
         like map and list, that read sequences.
         """
 
+
 class ISet(IKeySequence, ISetMutable):
     pass
 
+
 class ITreeSet(IKeyed, ISetMutable):
     pass
 
-class IMinimalDictionary(ISized):
-
-    def has_key(key):
-        """Check whether the object has an item with the given key.
-
-        Return a true value if so, else a false value.
-        """
-
+class IMinimalDictionary(ISized, IKeyed):
 
     def get(key, default):
         """Get the value associated with the given key.
@@ -138,69 +143,67 @@ class IMinimalDictionary(ISized):
     def __delitem__(key):
         """Delete the value associated with the given key.
 
-        Raise a KeyError if has_key(key) is false.
+        Raise KeyError if the key if has_key(key) is false.
         """
 
-    def values():
+    def values(min=None, max=None, excludemin=False, excludemax=False):
         """Return an IReadSequence containing the values in the collection.
 
-        The type of the IReadSequence is not specified.  It could be a
-        list or a tuple or some other type.
+        The type of the IReadSequence is not specified.  It could be a list
+        or a tuple or some other type.
+
+        All arguments are optional, and may be specified as keyword
+        arguments, or by position.
+
+        If a min is specified, then output is constrained to values whose
+        keys are greater than or equal to the given min, and, if excludemin
+        is specified and true, is further constrained to values whose keys
+        are strictly greater than min.  A min value of None is ignored.  If
+        min is None or not specified, and excludemin is true, the value
+        corresponding to the smallest key is excluded.
+
+        If a max is specified, then output is constrained to values whose
+        keys are less than or equal to the given max, and, if excludemax is
+        specified and true, is further constrained to values whose keys are
+        strictly less than max.  A max value of None is ignored.  If max is
+        None or not specified, and excludemax is true, the value
+        corresponding to the largest key is excluded.
         """
 
-    def keys():
-        """Return an IReadSequence containing the keys in the collection.
-
-        The type of the IReadSequence is not specified.  It could be a
-        list or a tuple or some other type.
-        """
-
-    def items():
+    def items(min=None, max=None, excludemin=False, excludemax=False):
         """Return an IReadSequence containing the items in the collection.
 
-        An item is a (key, value) 2-tuple.
+        An item is a 2-tuple, a (key, value) pair.
 
-        The type of the IReadSequence is not specified.  It could be a
-        list or a tuple or some other type.
+        The type of the IReadSequence is not specified.  It could be a list
+        or a tuple or some other type.
+
+        All arguments are optional, and may be specified as keyword
+        arguments, or by position.
+
+        If a min is specified, then output is constrained to items whose
+        keys are greater than or equal to the given min, and, if excludemin
+        is specified and true, is further constrained to items whose keys
+        are strictly greater than min.  A min value of None is ignored.  If
+        min is None or not specified, and excludemin is true, the item with
+        the smallest key is excluded.
+
+        If a max is specified, then output is constrained to items whose
+        keys are less than or equal to the given max, and, if excludemax is
+        specified and true, is further constrained to items whose keys are
+        strictly less than max.  A max value of None is ignored.  If max is
+        None or not specified, and excludemax is true, the item with the
+        largest key is excluded.
         """
 
-
-class IDictionaryIsh(IKeyed, IMinimalDictionary):
+class IDictionaryIsh(IMinimalDictionary):
 
     def update(collection):
         """Add the items from the given collection object to the collection.
 
         The input collection must be a sequence of (key, value) 2-tuples,
         or an object with an 'items' method that returns a sequence of
-        (key, value) 2-tuples.
-        """
-
-    def values(min=None, max=None):
-        """Return an IReadSequence containing the values in the collection.
-
-        The type of the IReadSequence is not specified.  It could be a
-        list or a tuple or some other type.
-
-        If a min is specified and not None, then output is constrained to
-        items having keys greater than or equal to the given min.
-
-        If a max is specified and not None, then output is constrained to
-        items having keys less than or equal to the given max.
-        """
-
-    def items(min=None, max=None):
-        """Return an IReadSequence containing the items in the collection.
-
-        An item is a (key, value) 2-tuple.
-
-        The type of the IReadSequence is not specified.  It could be a
-        list or a tuple or some other type.
-
-        If a min is specified and not None, then output is constrained to
-        items having keys greater than or equal to the given min.
-
-        If a max is specified and not None, then output is constrained to
-        items having keys less than or equal to the given min.
+        (key, value) pairs.
         """
 
     def byValue(minValue):
@@ -211,6 +214,25 @@ class IDictionaryIsh(IKeyed, IMinimalDictionary):
         integer values, the normalization is division.
         """
 
+    def setdefault(key, d):
+        """D.setdefault(k, d) -> D.get(k, d), also set D[k]=d if k not in D.
+
+        Return the value like get() except that if key is missing, d is both
+        returned and inserted into the dictionary as the value of k.
+
+        Note that, unlike as for Python's dict.setdefault(), d is not
+        optional.  Python defaults d to None, but that doesn't make sense
+        for mappings that can't have None as a value (for example, an
+        IIBTree can have only integers as values).
+        """
+
+    def pop(key, d):
+        """D.pop(k[, d]) -> v, remove key and return the corresponding value.
+
+        If key is not found, d is returned if given, otherwise KeyError is
+        raised.
+        """
+
 class IBTree(IDictionaryIsh):
 
     def insert(key, value):
@@ -232,6 +254,7 @@ class IBTree(IDictionaryIsh):
               key=generate_key()
         """
 
+
 class IMerge(Interface):
     """Object with methods for merging sets, buckets, and trees.
 
@@ -243,7 +266,7 @@ class IMerge(Interface):
 
     The implementing module has a value type. The IOBTree and OOBTree
     modules have object value type. The IIBTree and OIBTree modules
-    have integer value tyoes. Other modules may be defined in the
+    have integer value types. Other modules may be defined in the
     future that have other value types.
 
     The individual types are classified into set (Set and TreeSet) and
@@ -280,6 +303,7 @@ class IMerge(Interface):
         collections.
         """
 
+
 class IIMerge(IMerge):
     """Merge collections with integer value type.
 
@@ -319,7 +343,6 @@ class IIMerge(IMerge):
                   c2[key]  if the key is in c2 and c2 is a mapping
 
         Note that c1 and c2 must be collections.
-
         """
 
     def weightedIntersection(c1, c2, weight1=1, weight2=1):
@@ -352,6 +375,39 @@ class IIMerge(IMerge):
         Note that c1 and c2 must be collections.
         """
 
+
+class IMergeIntegerKey(IMerge):
+    """IMerge-able objects with integer keys.
+
+    Concretely, this means the types in IOBTree and IIBTree.
+    """
+
+    def multiunion(seq):
+        """Return union of (zero or more) integer sets, as an integer set.
+
+        seq is a sequence of objects each convertible to an integer set.
+        These objects are convertible to an integer set:
+
+        + An integer, which is added to the union.
+
+        + A Set or TreeSet from the same module (for example, an
+          IIBTree.TreeSet for IIBTree.multiunion()).  The elements of the
+          set are added to the union.
+
+        + A Bucket or BTree from the same module (for example, an
+          IOBTree.IOBTree for IOBTree.multiunion()).  The keys of the
+          mapping are added to the union.
+
+        The union is returned as a Set from the same module (for example,
+        IIBTree.multiunion() returns an IIBTree.IISet).
+
+        The point to this method is that it can run much faster than
+        doing a sequence of two-input union() calls.  Under the covers,
+        all the integers in all the inputs are sorted via a single
+        linear-time radix sort, then duplicates are removed in a second
+        linear-time pass.
+        """
+
 ###############################################################
 # IMPORTANT NOTE
 #
@@ -362,8 +418,3 @@ class IIMerge(IMerge):
 # Eventually, I need to express this through the interfaces.
 #
 ################################################################
-
-OOBTree.OOSet.__implements__=ISet
-OOBTree.OOTreeSet.__implements__=ITreeSet
-OOBTree.OOBucket.__implements__=IDictionaryIsh
-OOBTree.OOBTree.__implements__=IBTree
