@@ -4,7 +4,7 @@
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -12,24 +12,27 @@
 #
 ##############################################################################
 import os
+from unittest import TestCase, TestSuite, makeSuite
 
 from BTrees.OOBTree import OOBTree, OOBucket, OOSet, OOTreeSet
 from BTrees.IOBTree import IOBTree, IOBucket, IOSet, IOTreeSet
 from BTrees.IIBTree import IIBTree, IIBucket, IISet, IITreeSet
+from BTrees.IFBTree import IFBTree, IFBucket, IFSet, IFTreeSet
 from BTrees.OIBTree import OIBTree, OIBucket, OISet, OITreeSet
-from unittest import TestCase, TestSuite, makeSuite
 
+import transaction
 from ZODB.POSException import ConflictError
 
 class Base:
     """ Tests common to all types: sets, buckets, and BTrees """
 
-    db = None
+    storage = None
 
     def tearDown(self):
+        transaction.abort()
         del self.t
-        if self.db is not None:
-            self.db.close()
+        if self.storage is not None:
+            self.storage.close()
             self.storage.cleanup()
 
     def openDB(self):
@@ -74,19 +77,19 @@ class MappingBase(Base):
 
         r1 = self.db.open().root()
         r1["t"] = self.t
-        get_transaction().commit()
+        transaction.commit()
 
         r2 = self.db.open().root()
         copy = r2["t"]
-        list(copy.items())  # ensure it's all loaded
+        list(copy)    # unghostify
 
         self.assertEqual(self.t._p_serial, copy._p_serial)
 
         self.t.update({1:2, 2:3})
-        get_transaction().commit()
+        transaction.commit()
 
         copy.update({3:4})
-        get_transaction().commit()
+        transaction.commit()
 
 
     def testMergeDelete(self):
@@ -175,7 +178,7 @@ class MappingBase(Base):
 
         test_merge(base, b1, b2, bm, 'merge insert from empty')
 
-    def testMergeEmptyAndFill(self):
+    def testFailMergeEmptyAndFill(self):
         base, b1, b2, bm, e1, e2, items = self._setupConflict()
 
         b1.clear()
@@ -183,7 +186,7 @@ class MappingBase(Base):
         b2.update(e2)
         bm.update(e2)
 
-        test_merge(base, b1, b2, bm, 'merge insert from empty')
+        test_merge(base, b1, b2, bm, 'merge insert from empty', should_fail=1)
 
     def testMergeEmpty(self):
         base, b1, b2, bm, e1, e2, items = self._setupConflict()
@@ -202,7 +205,6 @@ class MappingBase(Base):
         test_merge(base, b1, b2, bm, 'merge conflicting inserts',
                    should_fail=1)
 
-
 class SetTests(Base):
     "Set (as opposed to TreeSet) specific tests."
 
@@ -215,13 +217,13 @@ class SetTests(Base):
         e2=[7745, 4868, -2548, -2711, -3154]
 
 
-        base = self.t
+        base=self.t
         base.update(l)
-        b1 = base.__class__(base.keys())
-        b2 = base.__class__(base.keys())
-        bm = base.__class__(base.keys())
+        b1=base.__class__(base)
+        b2=base.__class__(base)
+        bm=base.__class__(base)
 
-        items = base.keys()
+        items=base.keys()
 
         return  base, b1, b2, bm, e1, e2, items
 
@@ -273,7 +275,7 @@ class SetTests(Base):
 
         test_merge(base, b1, b2, bm, 'merge insert from empty')
 
-    def testMergeEmptyAndFill(self):
+    def testFailMergeEmptyAndFill(self):
         base, b1, b2, bm, e1, e2, items = self._setupConflict()
 
         b1.clear()
@@ -281,7 +283,7 @@ class SetTests(Base):
         b2.update(e2)
         bm.update(e2)
 
-        test_merge(base, b1, b2, bm, 'merge insert from empty')
+        test_merge(base, b1, b2, bm, 'merge insert from empty', should_fail=1)
 
     def testMergeEmpty(self):
         base, b1, b2, bm, e1, e2, items = self._setupConflict()
@@ -302,28 +304,27 @@ class SetTests(Base):
 
 
 def test_merge(o1, o2, o3, expect, message='failed to merge', should_fail=0):
-    s1=o1.__getstate__()
-    s2=o2.__getstate__()
-    s3=o3.__getstate__()
-    expected=expect.__getstate__()
+    s1 = o1.__getstate__()
+    s2 = o2.__getstate__()
+    s3 = o3.__getstate__()
+    expected = expect.__getstate__()
     if expected is None:
         expected = ((((),),),)
 
     if should_fail:
         try:
-            merged=o1._p_resolveConflict(s1, s2, s3)
+            merged = o1._p_resolveConflict(s1, s2, s3)
         except ConflictError, err:
             pass
         else:
             assert 0, message
     else:
-        merged=o1._p_resolveConflict(s1, s2, s3)
-        assert merged==expected, message
+        merged = o1._p_resolveConflict(s1, s2, s3)
+        assert merged == expected, message
 
 class BucketTests(MappingBase):
     """ Tests common to all buckets """
 
-
 class BTreeTests(MappingBase):
     """ Tests common to all BTrees """
 
@@ -345,6 +346,10 @@ class TestIIBTrees(BTreeTests, TestCase):
     def setUp(self):
         self.t = IIBTree()
 
+class TestIFBTrees(BTreeTests, TestCase):
+    def setUp(self):
+        self.t = IFBTree()
+
 ## Set tests
 
 class TestIOSets(SetTests, TestCase):
@@ -359,6 +364,10 @@ class TestIISets(SetTests, TestCase):
     def setUp(self):
         self.t = IISet()
 
+class TestIFSets(SetTests, TestCase):
+    def setUp(self):
+        self.t = IFSet()
+
 class TestOISets(SetTests, TestCase):
     def setUp(self):
         self.t = OISet()
@@ -375,6 +384,10 @@ class TestIITreeSets(SetTests, TestCase):
     def setUp(self):
         self.t = IITreeSet()
 
+class TestIFTreeSets(SetTests, TestCase):
+    def setUp(self):
+        self.t = IFTreeSet()
+
 class TestOITreeSets(SetTests, TestCase):
     def setUp(self):
         self.t = OITreeSet()
@@ -393,6 +406,10 @@ class TestIIBuckets(BucketTests, TestCase):
     def setUp(self):
         self.t = IIBucket()
 
+class TestIFBuckets(BucketTests, TestCase):
+    def setUp(self):
+        self.t = IFBucket()
+
 class TestOIBuckets(BucketTests, TestCase):
     def setUp(self):
         self.t = OIBucket()
@@ -428,7 +445,7 @@ class NastyConfict(Base, TestCase):
 
         r1 = self.db.open().root()
         r1["t"] = self.t
-        get_transaction().commit()
+        transaction.commit()
 
         r2 = self.db.open().root()
         copy = r2["t"]
@@ -438,10 +455,10 @@ class NastyConfict(Base, TestCase):
         self.assertEqual(self.t._p_serial, copy._p_serial)
 
         self.t.update({1:2, 2:3})
-        get_transaction().commit()
+        transaction.commit()
 
         copy.update({3:4})
-        get_transaction().commit()  # if this doesn't blow up
+        transaction.commit()  # if this doesn't blow up
         list(copy.values())         # and this doesn't either, then fine
 
     def testBucketSplitConflict(self):
@@ -468,9 +485,9 @@ class NastyConfict(Base, TestCase):
 
         r1 = self.db.open().root()
         r1["t"] = self.t
-        get_transaction().commit()
+        transaction.commit()
 
-        r2 = self.db.open().root()
+        r2 = self.db.open(synch=False).root()
         copy = r2["t"]
         # Make sure all of copy is loaded.
         list(copy.values())
@@ -500,7 +517,7 @@ class NastyConfict(Base, TestCase):
         self.assertEqual(state[0][3], 75)
         self.assertEqual(state[0][5], 120)
 
-        get_transaction().commit()
+        transaction.commit()
 
         # In the other transaction, add 3 values near the tail end of bucket1.
         # This doesn't cause a split.
@@ -518,8 +535,8 @@ class NastyConfict(Base, TestCase):
         self.assertEqual(state[0][1], 60)
         self.assertEqual(state[0][3], 120)
 
-        self.assertRaises(ConflictError, get_transaction().commit)
-        get_transaction().abort()   # horrible things happen w/o this
+        self.assertRaises(ConflictError, transaction.commit)
+        transaction.abort()   # horrible things happen w/o this
 
     def testEmptyBucketConflict(self):
         # Tests that an emptied bucket *created by* conflict resolution is
@@ -545,9 +562,9 @@ class NastyConfict(Base, TestCase):
 
         r1 = self.db.open().root()
         r1["t"] = self.t
-        get_transaction().commit()
+        transaction.commit()
 
-        r2 = self.db.open().root()
+        r2 = self.db.open(synch=False).root()
         copy = r2["t"]
         # Make sure all of copy is loaded.
         list(copy.values())
@@ -569,7 +586,7 @@ class NastyConfict(Base, TestCase):
         self.assertEqual(state[0][1], 60)
         self.assertEqual(state[0][3], 120)
 
-        get_transaction().commit()
+        transaction.commit()
 
         # In the other transaction, delete the other half of bucket 1.
         b = copy
@@ -590,8 +607,9 @@ class NastyConfict(Base, TestCase):
         # create an "insane" BTree (a legit BTree cannot contain an empty
         # bucket -- it contains NULL pointers the BTree code doesn't
         # expect, and segfaults result).
-        self.assertRaises(ConflictError, get_transaction().commit)
-        get_transaction().abort()   # horrible things happen w/o this
+        self.assertRaises(ConflictError, transaction.commit)
+        transaction.abort()   # horrible things happen w/o this
+
 
     def testEmptyBucketNoConflict(self):
         # Tests that a plain empty bucket (on input) is not viewed as a
@@ -616,7 +634,7 @@ class NastyConfict(Base, TestCase):
 
         r1 = self.db.open().root()
         r1["t"] = self.t
-        get_transaction().commit()
+        transaction.commit()
 
         r2 = self.db.open().root()
         copy = r2["t"]
@@ -639,7 +657,7 @@ class NastyConfict(Base, TestCase):
         self.assertEqual(state[0][1], 60)
         self.assertEqual(state[0][3], 120)
 
-        get_transaction().commit()
+        transaction.commit()
 
         # In the other transaction, delete bucket 2.
         b = copy
@@ -655,11 +673,10 @@ class NastyConfict(Base, TestCase):
         self.assertEqual(state[0][1], 60)
 
         # This shouldn't create a ConflictError.
-        get_transaction().commit()
+        transaction.commit()
         # And the resulting BTree shouldn't have internal damage.
         b._check()
 
-
     # The snaky control flow in _bucket__p_resolveConflict ended up trying
     # to decref a NULL pointer if conflict resolution was fed 3 empty
     # buckets.  http://collector.zope.org/Zope/553
@@ -669,9 +686,9 @@ class NastyConfict(Base, TestCase):
         r1 = self.db.open().root()
         self.assertEqual(len(self.t), 0)
         r1["t"] = b = self.t  # an empty tree
-        get_transaction().commit()
+        transaction.commit()
 
-        r2 = self.db.open().root()
+        r2 = self.db.open(synch=False).root()
         copy = r2["t"]
         # Make sure all of copy is loaded.
         list(copy.values())
@@ -679,22 +696,162 @@ class NastyConfict(Base, TestCase):
         # In one transaction, add and delete a key.
         b[2] = 2
         del b[2]
-        get_transaction().commit()
+        transaction.commit()
 
         # In the other transaction, also add and delete a key.
         b = copy
         b[1] = 1
         del b[1]
         # If the commit() segfaults, the C code is still wrong for this case.
-        self.assertRaises(ConflictError, get_transaction().commit)
-        get_transaction().abort()
+        self.assertRaises(ConflictError, transaction.commit)
+        transaction.abort()
+
+    def testCantResolveBTreeConflict(self):
+        # Test that a conflict involving two different changes to
+        # an internal BTree node is unresolvable.  An internal node
+        # only changes when there are enough additions or deletions
+        # to a child bucket that the bucket is split or removed.
+        # It's (almost necessarily) a white-box test, and sensitive to
+        # implementation details.
+        b = self.t
+        for i in range(0, 200, 4):
+            b[i] = i
+        # bucket 0 has 15 values: 0, 4 .. 56
+        # bucket 1 has 15 values: 60, 64 .. 116
+        # bucket 2 has 20 values: 120, 124 .. 196
+        state = b.__getstate__()
+        # Looks like:  ((bucket0, 60, bucket1, 120, bucket2), firstbucket)
+        # If these fail, the *preconditions* for running the test aren't
+        # satisfied -- the test itself hasn't been run yet.
+        self.assertEqual(len(state), 2)
+        self.assertEqual(len(state[0]), 5)
+        self.assertEqual(state[0][1], 60)
+        self.assertEqual(state[0][3], 120)
+
+        # Set up database connections to provoke conflict.
+        self.openDB()
+        r1 = self.db.open().root()
+        r1["t"] = self.t
+        transaction.commit()
+
+        r2 = self.db.open(synch=False).root()
+        copy = r2["t"]
+        # Make sure all of copy is loaded.
+        list(copy.values())
+
+        self.assertEqual(self.t._p_serial, copy._p_serial)
+
+        # Now one transaction should add enough keys to cause a split,
+        # and another should remove all the keys in one bucket.
+
+        for k in range(200, 300, 4):
+            self.t[k] = k
+        transaction.commit()
+
+        for k in range(0, 60, 4):
+            del copy[k]
+
+        try:
+            transaction.commit()
+        except ConflictError, detail:
+            self.assert_(str(detail).startswith('database conflict error'))
+            transaction.abort()
+        else:
+            self.fail("expected ConflictError")
+
+    def testConflictWithOneEmptyBucket(self):
+        # If one transaction empties a bucket, while another adds an item
+        # to the bucket, all the changes "look resolvable":  bucket conflict
+        # resolution returns a bucket containing (only) the item added by
+        # the latter transaction, but changes from the former transaction
+        # removing the bucket are uncontested:  the bucket is removed from
+        # the BTree despite that resolution thinks it's non-empty!  This
+        # was first reported by Dieter Maurer, to zodb-dev on 22 Mar 2005.
+        b = self.t
+        for i in range(0, 200, 4):
+            b[i] = i
+        # bucket 0 has 15 values: 0, 4 .. 56
+        # bucket 1 has 15 values: 60, 64 .. 116
+        # bucket 2 has 20 values: 120, 124 .. 196
+        state = b.__getstate__()
+        # Looks like:  ((bucket0, 60, bucket1, 120, bucket2), firstbucket)
+        # If these fail, the *preconditions* for running the test aren't
+        # satisfied -- the test itself hasn't been run yet.
+        self.assertEqual(len(state), 2)
+        self.assertEqual(len(state[0]), 5)
+        self.assertEqual(state[0][1], 60)
+        self.assertEqual(state[0][3], 120)
+
+        # Set up database connections to provoke conflict.
+        self.openDB()
+        r1 = self.db.open().root()
+        r1["t"] = self.t
+        transaction.commit()
+
+        r2 = self.db.open(synch=False).root()
+        copy = r2["t"]
+        # Make sure all of copy is loaded.
+        list(copy.values())
+
+        self.assertEqual(self.t._p_serial, copy._p_serial)
+
+        # Now one transaction empties the first bucket, and another adds a
+        # key to the first bucket.
+
+        for k in range(0, 60, 4):
+            del self.t[k]
+        transaction.commit()
+
+        copy[1] = 1
+
+        try:
+            transaction.commit()
+        except ConflictError, detail:
+            self.assert_(str(detail).startswith('database conflict error'))
+            transaction.abort()
+        else:
+            self.fail("expected ConflictError")
+
+        # Same thing, except commit the transactions in the opposite order.
+        b = OOBTree()
+        for i in range(0, 200, 4):
+            b[i] = i
+
+        r1 = self.db.open().root()
+        r1["t"] = b
+        transaction.commit()
+
+        r2 = self.db.open(synch=False).root()
+        copy = r2["t"]
+        # Make sure all of copy is loaded.
+        list(copy.values())
+
+        self.assertEqual(b._p_serial, copy._p_serial)
+
+        # Now one transaction empties the first bucket, and another adds a
+        # key to the first bucket.
+        b[1] = 1
+        transaction.commit()
+
+        for k in range(0, 60, 4):
+            del copy[k]
+        try:
+            transaction.commit()
+        except ConflictError, detail:
+            self.assert_(str(detail).startswith('database conflict error'))
+            transaction.abort()
+        else:
+            self.fail("expected ConflictError")
+
 
 def test_suite():
     suite = TestSuite()
-    for k in (TestIOBTrees,   TestOOBTrees,   TestOIBTrees,   TestIIBTrees,
-              TestIOSets,     TestOOSets,     TestOISets,     TestIISets,
-              TestIOTreeSets, TestOOTreeSets, TestOITreeSets, TestIITreeSets,
-              TestIOBuckets,  TestOOBuckets,  TestOIBuckets,  TestIIBuckets,
-              NastyConfict):
+    for k in (
+        TestIIBTrees, TestIISets, TestIITreeSets, TestIIBuckets,
+        TestIFBTrees, TestIFSets, TestIFTreeSets, TestIFBuckets,
+        TestIOBTrees, TestIOSets, TestIOTreeSets, TestIOBuckets,
+        TestOOBTrees, TestOOSets, TestOOTreeSets, TestOOBuckets,
+        TestOIBTrees, TestOISets, TestOITreeSets, TestOIBuckets,
+        NastyConfict):
         suite.addTest(makeSuite(k))
     return suite
