@@ -1,35 +1,51 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-# FOR A PARTICULAR PURPOSE
+# FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
 """Copy interface
 
-$Id: CopySupport.py 40550 2005-12-05 18:26:15Z yuppie $
+$Id: CopySupport.py 81167 2007-10-28 09:40:32Z hannosch $
 """
 
 import re, sys, tempfile
+import warnings
 from cgi import escape
 from marshal import loads, dumps
 from urllib import quote, unquote
-from warnings import warn
 from zlib import compress, decompress
 
 import Globals, Moniker, ExtensionClass
 import transaction
-from App.Dialogs import MessageDialog
+from Globals import InitializeClass
+from AccessControl import ClassSecurityInfo
 from AccessControl import getSecurityManager
-from AccessControl.Permissions import delete_objects as DeleteObjects
+from AccessControl.Permissions import view_management_screens
+from AccessControl.Permissions import copy_or_move
+from AccessControl.Permissions import delete_objects
 from Acquisition import aq_base, aq_inner, aq_parent
-from zExceptions import Unauthorized, BadRequest
+from App.Dialogs import MessageDialog
 from webdav.Lockable import ResourceLockedError
+from zExceptions import Unauthorized, BadRequest
+from ZODB.POSException import ConflictError
+from zope.interface import implements
+from zope.event import notify
+from zope.lifecycleevent import ObjectCopiedEvent
+from zope.app.container.contained import ObjectMovedEvent
+from zope.app.container.contained import notifyContainerModified
+from OFS.event import ObjectWillBeMovedEvent
+from OFS.event import ObjectClonedEvent
+import OFS.subscribers
+
+from OFS.interfaces import ICopyContainer
+from OFS.interfaces import ICopySource
 
 
 CopyError='Copy Error'
@@ -43,14 +59,9 @@ class CopyContainer(ExtensionClass.Base):
 
     """Interface for containerish objects which allow cut/copy/paste"""
 
-    __ac_permissions__=(
-        ('View management screens',
-         ('manage_copyObjects', 'manage_pasteObjects',
-          'manage_renameForm', 'manage_renameObject', 'manage_renameObjects',)),
-        ('Delete objects',
-         ('manage_cutObjects',)),
-        )
+    implements(ICopyContainer)
 
+    security = ClassSecurityInfo()
 
     # The following three methods should be overridden to store sub-objects
     # as non-attributes.
@@ -73,6 +84,7 @@ class CopyContainer(ExtensionClass.Base):
     def manage_CopyContainerAllItems(self, REQUEST):
         return map(lambda i, s=self: s._getOb(i), tuple(REQUEST['ids']))
 
+    security.declareProtected(delete_objects, 'manage_cutObjects')
     def manage_cutObjects(self, ids=None, REQUEST=None):
         """Put a reference to the objects named in ids in the clip board"""
         if ids is None and REQUEST is not None:
@@ -102,6 +114,7 @@ class CopyContainer(ExtensionClass.Base):
             return self.manage_main(self, REQUEST)
         return cp
 
+    security.declareProtected(view_management_screens, 'manage_copyObjects')
     def manage_copyObjects(self, ids=None, REQUEST=None, RESPONSE=None):
         """Put a reference to the objects named in ids in the clip board"""
         if ids is None and REQUEST is not None:
@@ -144,91 +157,144 @@ class CopyContainer(ExtensionClass.Base):
             id='copy%s_of_%s' % (n and n+1 or '', orig_id)
             n=n+1
 
+    security.declareProtected(view_management_screens, 'manage_pasteObjects')
     def manage_pasteObjects(self, cb_copy_data=None, REQUEST=None):
         """Paste previously copied objects into the current object.
 
         If calling manage_pasteObjects from python code, pass the result of a
         previous call to manage_cutObjects or manage_copyObjects as the first
         argument.
+
+        Also sends IObjectCopiedEvent and IObjectClonedEvent
+        or IObjectWillBeMovedEvent and IObjectMovedEvent.
         """
-        cp=None
         if cb_copy_data is not None:
-            cp=cb_copy_data
+            cp = cb_copy_data
+        elif REQUEST is not None and REQUEST.has_key('__cp'):
+            cp = REQUEST['__cp']
         else:
-            if REQUEST and REQUEST.has_key('__cp'):
-                cp=REQUEST['__cp']
+            cp = None
         if cp is None:
             raise CopyError, eNoData
 
-        try:    cp=_cb_decode(cp)
-        except: raise CopyError, eInvalid
+        try:
+            op, mdatas = _cb_decode(cp)
+        except:
+            raise CopyError, eInvalid
 
-        oblist=[]
-        op=cp[0]
+        oblist = []
         app = self.getPhysicalRoot()
-        result = []
-
-        for mdata in cp[1]:
+        for mdata in mdatas:
             m = Moniker.loadMoniker(mdata)
-            try: ob = m.bind(app)
-            except: raise CopyError, eNotFound
+            try:
+                ob = m.bind(app)
+            except ConflictError:
+                raise
+            except:
+                raise CopyError, eNotFound
             self._verifyObjectPaste(ob, validate_src=op+1)
             oblist.append(ob)
 
-        if op==0:
+        result = []
+        if op == 0:
             # Copy operation
             for ob in oblist:
+                orig_id = ob.getId()
                 if not ob.cb_isCopyable():
-                    raise CopyError, eNotSupported % escape(ob.getId())
-                try:    ob._notifyOfCopyTo(self, op=0)
-                except: raise CopyError, MessageDialog(
-                    title='Copy Error',
-                    message=sys.exc_info()[1],
-                    action ='manage_main')
-                ob=ob._getCopy(self)
-                orig_id=ob.getId()
-                id=self._get_id(ob.getId())
-                result.append({'id':orig_id, 'new_id':id})
+                    raise CopyError, eNotSupported % escape(orig_id)
+
+                try:
+                    ob._notifyOfCopyTo(self, op=0)
+                except ConflictError:
+                    raise
+                except:
+                    raise CopyError, MessageDialog(
+                        title="Copy Error",
+                        message=sys.exc_info()[1],
+                        action='manage_main')
+
+                id = self._get_id(orig_id)
+                result.append({'id': orig_id, 'new_id': id})
+
+                orig_ob = ob
+                ob = ob._getCopy(self)
                 ob._setId(id)
+                notify(ObjectCopiedEvent(ob, orig_ob))
+
                 self._setObject(id, ob)
                 ob = self._getOb(id)
-                ob._postCopy(self, op=0)
-                ob.manage_afterClone(ob)
                 ob.wl_clearLocks()
 
+                ob._postCopy(self, op=0)
+
+                OFS.subscribers.compatibilityCall('manage_afterClone', ob, ob)
+
+                notify(ObjectClonedEvent(ob))
+
             if REQUEST is not None:
                 return self.manage_main(self, REQUEST, update_menu=1,
                                         cb_dataValid=1)
 
-        if op==1:
+        elif op == 1:
             # Move operation
             for ob in oblist:
-                id=ob.getId()
+                orig_id = ob.getId()
                 if not ob.cb_isMoveable():
-                    raise CopyError, eNotSupported % escape(id)
-                try:    ob._notifyOfCopyTo(self, op=1)
-                except: raise CopyError, MessageDialog(
-                    title='Move Error',
-                    message=sys.exc_info()[1],
-                    action ='manage_main')
+                    raise CopyError, eNotSupported % escape(orig_id)
+
+                try:
+                    ob._notifyOfCopyTo(self, op=1)
+                except ConflictError:
+                    raise
+                except:
+                    raise CopyError, MessageDialog(
+                        title="Move Error",
+                        message=sys.exc_info()[1],
+                        action='manage_main')
+
                 if not sanity_check(self, ob):
-                    raise CopyError, 'This object cannot be pasted into itself'
+                    raise CopyError, "This object cannot be pasted into itself"
+
+                orig_container = aq_parent(aq_inner(ob))
+                if aq_base(orig_container) is aq_base(self):
+                    id = orig_id
+                else:
+                    id = self._get_id(orig_id)
+                result.append({'id': orig_id, 'new_id': id})
+
+                notify(ObjectWillBeMovedEvent(ob, orig_container, orig_id,
+                                              self, id))
 
                 # try to make ownership explicit so that it gets carried
                 # along to the new location if needed.
                 ob.manage_changeOwnershipType(explicit=1)
 
-                aq_parent(aq_inner(ob))._delObject(id)
+                try:
+                    orig_container._delObject(orig_id, suppress_events=True)
+                except TypeError:
+                    orig_container._delObject(orig_id)
+                    warnings.warn(
+                        "%s._delObject without suppress_events is discouraged."
+                        % orig_container.__class__.__name__,
+                        DeprecationWarning)
                 ob = aq_base(ob)
-                orig_id=id
-                id=self._get_id(id)
-                result.append({'id':orig_id, 'new_id':id })
-
                 ob._setId(id)
-                self._setObject(id, ob, set_owner=0)
-                ob=self._getOb(id)
-                ob._postCopy(self, op=1)
 
+                try:
+                    self._setObject(id, ob, set_owner=0, suppress_events=True)
+                except TypeError:
+                    self._setObject(id, ob, set_owner=0)
+                    warnings.warn(
+                        "%s._setObject without suppress_events is discouraged."
+                        % self.__class__.__name__, DeprecationWarning)
+                ob = self._getOb(id)
+
+                notify(ObjectMovedEvent(ob, orig_container, orig_id, self, id))
+                notifyContainerModified(orig_container)
+                if aq_base(orig_container) is not aq_base(self):
+                    notifyContainerModified(self)
+
+                ob._postCopy(self, op=1)
                 # try to make ownership implicit if possible
                 ob.manage_changeOwnershipType(explicit=0)
 
@@ -239,11 +305,13 @@ class CopyContainer(ExtensionClass.Base):
                 REQUEST['__cp'] = None
                 return self.manage_main(self, REQUEST, update_menu=1,
                                         cb_dataValid=0)
+
         return result
 
-
+    security.declareProtected(view_management_screens, 'manage_renameForm')
     manage_renameForm=Globals.DTMLFile('dtml/renameForm', globals())
 
+    security.declareProtected(view_management_screens, 'manage_renameObjects')
     def manage_renameObjects(self, ids=[], new_ids=[], REQUEST=None):
         """Rename several sub-objects"""
         if len(ids) != len(new_ids):
@@ -255,32 +323,63 @@ class CopyContainer(ExtensionClass.Base):
             return self.manage_main(self, REQUEST, update_menu=1)
         return None
 
+    security.declareProtected(view_management_screens, 'manage_renameObject')
     def manage_renameObject(self, id, new_id, REQUEST=None):
-        """Rename a particular sub-object"""
-        try: self._checkId(new_id)
-        except: raise CopyError, MessageDialog(
-                      title='Invalid Id',
-                      message=sys.exc_info()[1],
-                      action ='manage_main')
-        ob=self._getOb(id)
+        """Rename a particular sub-object.
+        """
+        try:
+            self._checkId(new_id)
+        except:
+            raise CopyError, MessageDialog(
+                title='Invalid Id',
+                message=sys.exc_info()[1],
+                action ='manage_main')
+
+        ob = self._getOb(id)
+
         if ob.wl_isLocked():
-            raise ResourceLockedError, 'Object "%s" is locked via WebDAV' % ob.getId()
+            raise ResourceLockedError, ('Object "%s" is locked via WebDAV'
+                                        % ob.getId())
         if not ob.cb_isMoveable():
             raise CopyError, eNotSupported % escape(id)
         self._verifyObjectPaste(ob)
-        try:    ob._notifyOfCopyTo(self, op=1)
-        except: raise CopyError, MessageDialog(
-                      title='Rename Error',
-                      message=sys.exc_info()[1],
-                      action ='manage_main')
-        self._delObject(id)
+
+        try:
+            ob._notifyOfCopyTo(self, op=1)
+        except ConflictError:
+            raise
+        except:
+            raise CopyError, MessageDialog(
+                title="Rename Error",
+                message=sys.exc_info()[1],
+                action ='manage_main')
+
+        notify(ObjectWillBeMovedEvent(ob, self, id, self, new_id))
+
+        try:
+            self._delObject(id, suppress_events=True)
+        except TypeError:
+            self._delObject(id)
+            warnings.warn(
+                "%s._delObject without suppress_events is discouraged." %
+                self.__class__.__name__, DeprecationWarning)
         ob = aq_base(ob)
         ob._setId(new_id)
 
         # Note - because a rename always keeps the same context, we
         # can just leave the ownership info unchanged.
-        self._setObject(new_id, ob, set_owner=0)
+        try:
+            self._setObject(new_id, ob, set_owner=0, suppress_events=True)
+        except TypeError:
+            self._setObject(new_id, ob, set_owner=0)
+            warnings.warn(
+                "%s._setObject without suppress_events is discouraged." %
+                self.__class__.__name__, DeprecationWarning)
         ob = self._getOb(new_id)
+
+        notify(ObjectMovedEvent(ob, self, id, self, new_id))
+        notifyContainerModified(self)
+
         ob._postCopy(self, op=1)
 
         if REQUEST is not None:
@@ -291,28 +390,47 @@ class CopyContainer(ExtensionClass.Base):
     # supposed to be public since it does its own auth ?
     #
     # Because it's still a "management" function.
-    manage_clone__roles__=None
+
+    security.declarePublic('manage_clone')
     def manage_clone(self, ob, id, REQUEST=None):
-        # Clone an object, creating a new object with the given id.
+        """Clone an object, creating a new object with the given id.
+        """
         if not ob.cb_isCopyable():
             raise CopyError, eNotSupported % escape(ob.getId())
-        try: self._checkId(id)
-        except: raise CopyError, MessageDialog(
-                      title='Invalid Id',
-                      message=sys.exc_info()[1],
-                      action ='manage_main')
+        try:
+            self._checkId(id)
+        except:
+            raise CopyError, MessageDialog(
+                title='Invalid Id',
+                message=sys.exc_info()[1],
+                action ='manage_main')
+
         self._verifyObjectPaste(ob)
-        try:    ob._notifyOfCopyTo(self, op=0)
-        except: raise CopyError, MessageDialog(
-                      title='Clone Error',
-                      message=sys.exc_info()[1],
-                      action ='manage_main')
-        ob=ob._getCopy(self)
+
+        try:
+            ob._notifyOfCopyTo(self, op=0)
+        except ConflictError:
+            raise
+        except:
+            raise CopyError, MessageDialog(
+                title="Clone Error",
+                message=sys.exc_info()[1],
+                action='manage_main')
+
+        orig_ob = ob
+        ob = ob._getCopy(self)
         ob._setId(id)
+        notify(ObjectCopiedEvent(ob, orig_ob))
+
         self._setObject(id, ob)
-        ob=self._getOb(id)
+        ob = self._getOb(id)
+
         ob._postCopy(self, op=0)
-        ob.manage_afterClone(ob)
+
+        OFS.subscribers.compatibilityCall('manage_afterClone', ob, ob)
+
+        notify(ObjectClonedEvent(ob))
+
         return ob
 
     def cb_dataValid(self):
@@ -389,7 +507,7 @@ class CopyContainer(ExtensionClass.Base):
                         raise Unauthorized(absattr(object.id))
 
                     if validate_src == 2: # moving
-                        if not sm.checkPermission(DeleteObjects, parent):
+                        if not sm.checkPermission(delete_objects, parent):
                             raise Unauthorized('Delete not allowed.')
             else:
                 raise CopyError, MessageDialog(
@@ -399,42 +517,6 @@ class CopyContainer(ExtensionClass.Base):
                                'pasting, thus you are not able to perform '
                                'this operation.' % mt_permission),
                     action = 'manage_main')
-        elif method_name:
-            # BBB: fallback for missing or None permission
-            warn("The required 'permission' key is not set or None for meta "
-                 "type '%s'. This fallback will be removed in Zope 2.9."
-                 % object.meta_type,
-                 DeprecationWarning)
-            try:
-                method = self.restrictedTraverse(method_name)
-            except Unauthorized:
-                raise CopyError, MessageDialog(
-                    title = 'Insufficient Privileges',
-                    message = ('You do not possess the permission required '
-                               'to call %s in the context of the container '
-                               'into which you are pasting, thus you are not '
-                               'able to perform this operation.'
-                               % method_name),
-                    action = 'manage_main')
-
-            if validate_src:
-
-                sm = getSecurityManager()
-
-                # Ensure the user is allowed to access the object on the
-                # clipboard.
-                try:
-                    parent = aq_parent(aq_inner(object))
-                except:
-                    parent = None
-
-                if not sm.validate(None,parent,None,object):
-                    raise Unauthorized, absattr(object.id)
-
-                if validate_src == 2: # moving
-                    if not sm.checkPermission(DeleteObjects, parent):
-                        raise Unauthorized, 'Delete not allowed.'
-
         else:
             raise CopyError, MessageDialog(
                 title = 'Not Supported',
@@ -442,19 +524,19 @@ class CopyContainer(ExtensionClass.Base):
                            'operation.' % escape(absattr(object.id))),
                 action = 'manage_main')
 
-Globals.default__class_init__(CopyContainer)
-
+InitializeClass(CopyContainer)
 
 
 class CopySource(ExtensionClass.Base):
 
     """Interface for objects which allow themselves to be copied."""
 
+    implements(ICopySource)
+
     # declare a dummy permission for Copy or Move here that we check
     # in cb_isCopyable.
-    __ac_permissions__=(
-        ('Copy or Move', (), ('Anonymous', 'Manager',)),
-        )
+    security = ClassSecurityInfo()
+    security.setPermissionDefault(copy_or_move, ('Anonymous', 'Manager'))
 
     def _canCopy(self, op=0):
         """Called to make sure this object is copyable.
@@ -527,10 +609,10 @@ class CopySource(ExtensionClass.Base):
         return 1
 
     def cb_userHasCopyOrMovePermission(self):
-        if getSecurityManager().checkPermission('Copy or Move', self):
+        if getSecurityManager().checkPermission(copy_or_move, self):
             return 1
 
-Globals.default__class_init__(CopySource)
+InitializeClass(CopySource)
 
 
 def sanity_check(c, ob):
