@@ -252,7 +252,12 @@ class FileTests(unittest.TestCase):
         verifyClass(HTTPRangeInterface, File)
         verifyClass(WriteLockInterface, File)
 
-
+    def testUnicode(self):
+        val = u'some unicode string here'
+        
+        self.assertRaises(TypeError, self.file.manage_edit,
+                          'foobar', 'text/plain', filedata=val)
+        
 class ImageTests(FileTests):
     data = open(filedata, 'rb').read()
     content_type = 'image/gif'
@@ -272,8 +277,12 @@ class ImageTests(FileTests):
           ('<img src="http://foo/file" alt="" title="" height="16" width="16" />'))
 
     def testTag(self):
-        self.assertEqual(self.file.tag(),
-          ('<img src="http://foo/file" alt="" title="" height="16" width="16" />'))
+        tag_fmt = '<img src="http://foo/file" alt="%s" title="%s" height="16" width="16" />'
+        self.assertEqual(self.file.tag(), (tag_fmt % ('','')))
+        self.file.manage_changeProperties(title='foo')
+        self.assertEqual(self.file.tag(), (tag_fmt % ('','foo')))
+        self.file.manage_changeProperties(alt='bar')
+        self.assertEqual(self.file.tag(), (tag_fmt % ('bar','foo')))
 
     def testViewImageOrFile(self):
         pass # dtml method,screw it
@@ -285,7 +294,6 @@ class ImageTests(FileTests):
 
         verifyClass(WriteLockInterface, Image)
 
-
 def test_suite():
     return unittest.TestSuite((
         unittest.makeSuite(FileTests),
