@@ -1,9 +1,9 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -11,16 +11,16 @@
 #
 ##############################################################################
 """
-$Id: ProgressHandler.py 38741 2005-10-04 14:43:25Z yuppie $
+$Id: ProgressHandler.py 41191 2006-01-08 10:23:26Z andreasjung $
 """
 
 import time, sys
-from zLOG import LOG, INFO
+from logging import getLogger
 
 from DateTime.DateTime import DateTime
-
 from Interface import Interface
 
+LOG = getLogger('ProgressHandler')
 
 class IProgressHandler(Interface):
     """ A handler to log progress informations for long running 
@@ -87,12 +87,12 @@ class StdoutHandler:
 
 
 class ZLogHandler(StdoutHandler):
-    """ Use zLOG """
+    """ Use Zope logger"""
 
     __implements__ = IProgressHandler
 
     def output(self, text):
-        LOG(self._ident, INFO, text)
+        LOG.info(text)
 
 
 class FilelogHandler(StdoutHandler):
