@@ -15,7 +15,7 @@
 These functions are designed to be imported and run at
 module level to add functionality to the test environment.
 
-$Id: utils.py 66621 2006-04-06 21:30:58Z slinkp $
+$Id: utils.py 73192 2007-03-15 14:20:14Z shh $
 """
 
 import os
@@ -115,7 +115,7 @@ def startZServer(number_of_threads=1, log=None):
     if _Z2HOST is None:
         _Z2HOST = '127.0.0.1'
         _Z2PORT = random.choice(range(55000, 55500))
-        from ZServer import setNumberOfThreads
+        from threadutils import setNumberOfThreads
         setNumberOfThreads(number_of_threads)
         from threadutils import QuietThread, zserverRunner
         t = QuietThread(target=zserverRunner, args=(_Z2HOST, _Z2PORT, log))
@@ -127,24 +127,12 @@ def startZServer(number_of_threads=1, log=None):
 
 def makerequest(app, stdout=sys.stdout):
     '''Wraps the app into a fresh REQUEST.'''
-    from ZPublisher.BaseRequest import RequestContainer
-    from ZPublisher.Request import Request
-    from ZPublisher.Response import Response
-    response = Response(stdout=stdout)
+    from Testing.makerequest import makerequest as _makerequest
     environ = {}
     environ['SERVER_NAME'] = _Z2HOST or 'nohost'
     environ['SERVER_PORT'] = '%d' % (_Z2PORT or 80)
     environ['REQUEST_METHOD'] = 'GET'
-    request = Request(sys.stdin, environ, response)
-    request._steps = ['noobject'] # Fake a published object
-    request['ACTUAL_URL'] = request.get('URL') # Zope 2.7.4
-
-    # set Zope3-style default skin so that the request is usable for
-    # Zope3-style view look-ups
-    from zope.app.publication.browser import setDefaultSkin
-    setDefaultSkin(request)
-
-    return app.__of__(RequestContainer(REQUEST=request))
+    return _makerequest(app, stdout=stdout, environ=environ)
 
 
 def appcall(function, *args, **kw):
