@@ -17,14 +17,12 @@ MappingStorage.  Unlike MappingStorage, it needs not be packed to get rid of
 non-cyclic garbage and it does rudimentary conflict resolution.  This is a
 ripoff of Jim's Packless bsddb3 storage.
 
-$Id: TemporaryStorage.py 68835 2006-06-24 22:45:20Z tseaver $
+$Id: TemporaryStorage.py 41192 2006-01-08 10:25:23Z andreasjung $
 """
 
 __version__ ='$Revision: 1.1.2.2 $'[11:-2]
-import logging
-logger = logging.getLogger('TemporaryStorage')
 
-from zLOG.EventLogger import CUSTOM_BLATHER
+from logging import getLogger
 from ZODB.serialize import referencesf
 from ZODB import POSException
 from ZODB.BaseStorage import BaseStorage
@@ -39,6 +37,8 @@ CONFLICT_CACHE_GCEVERY = 60
 # keep history of recently gc'ed oids of length RECENTLY_GC_OIDS_LEN
 RECENTLY_GC_OIDS_LEN = 200
 
+LOG = getLogger('TemporaryStorage')
+
 class ReferenceCountError(POSException.POSError):
     """ An error occured while decrementing a reference to an object in
     the commit phase. The object's reference count was below zero."""
@@ -184,10 +184,9 @@ class TemporaryStorage(BaseStorage, ConflictResolvingStorage):
         if version:
             # we allow a version to be in use although we don't
             # support versions in the storage.
-            logging.log(CUSTOM_BLATHER,
-                ('versions in use with TemporaryStorage although Temporary'
-                 'Storage doesnt support versions'),
-                )
+            LOG.debug('versions in use with TemporaryStorage although Temporary '
+                      'Storage doesnt support versions')
+
         self._lock_acquire()
         try:
             if self._index.has_key(oid):
