@@ -12,17 +12,27 @@
 ##############################################################################
 """WebDAV support - resource objects.
 
-$Id: Resource.py 69723 2006-08-22 10:05:31Z shh $
+$Id: Resource.py 77126 2007-06-27 12:59:31Z sidnei $
 """
 
 import mimetypes
 import sys
+import warnings
+import re
 from urllib import unquote
 
+import webdav
 import ExtensionClass
-import Globals
+from Globals import InitializeClass
 from AccessControl import getSecurityManager
-from Acquisition import aq_base
+from AccessControl import ClassSecurityInfo
+from AccessControl.Permissions import delete_objects
+from AccessControl.Permissions import manage_properties
+from AccessControl.Permissions import view as View
+from AccessControl.Permissions import webdav_lock_items
+from AccessControl.Permissions import webdav_unlock_items
+from AccessControl.Permissions import webdav_access
+from Acquisition import aq_base, aq_inner, aq_parent
 from zExceptions import BadRequest, MethodNotAllowed
 from zExceptions import Unauthorized, Forbidden, NotFound
 from zope.interface import implements
@@ -39,9 +49,14 @@ from interfaces import IWriteLock
 from WriteLockInterface import WriteLockInterface
 
 from zope.event import notify
+from zope.lifecycleevent import ObjectCopiedEvent
+from zope.app.container.contained import ObjectMovedEvent
+from zope.app.container.contained import notifyContainerModified
 from OFS.event import ObjectClonedEvent
+from OFS.event import ObjectWillBeMovedEvent
 import OFS.subscribers
 
+ms_dav_agent = re.compile("Microsoft.*Internet Publishing.*")
 
 class Resource(ExtensionClass.Base, Lockable.LockableItem):
 
@@ -60,16 +75,8 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
                       'MOVE', 'LOCK', 'UNLOCK',
                       )
 
-    __ac_permissions__=(
-        ('View',                             ('HEAD',)),
-        ('WebDAV access',                    ('PROPFIND', 'manage_DAVget',
-                                              'listDAVObjects'),
-         ('Authenticated', 'Manager')),
-        ('Manage properties',                ('PROPPATCH',)),
-        ('Delete objects',                   ('DELETE',)),
-        ('WebDAV Lock items',                ('LOCK',)),
-        ('WebDAV Unlock items',              ('UNLOCK',)),
-    )
+    security = ClassSecurityInfo()
+    security.setPermissionDefault(webdav_access, ('Authenticated', 'Manager'))
 
     def dav__init(self, request, response):
         # Init expected HTTP 1.1 / WebDAV headers which are not
@@ -162,6 +169,7 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
 
 
     # WebDAV class 1 support
+    security.declareProtected(View, 'HEAD')
     def HEAD(self, REQUEST, RESPONSE):
         """Retrieve resource information without a response body."""
         self.dav__init(REQUEST, RESPONSE)
@@ -201,17 +209,26 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
         self.dav__init(REQUEST, RESPONSE)
         raise MethodNotAllowed, 'Method not supported for this resource.'
 
-    OPTIONS__roles__=None
+    security.declarePublic('OPTIONS')
     def OPTIONS(self, REQUEST, RESPONSE):
         """Retrieve communication options."""
         self.dav__init(REQUEST, RESPONSE)
         RESPONSE.setHeader('Allow', ', '.join(self.__http_methods__))
         RESPONSE.setHeader('Content-Length', 0)
         RESPONSE.setHeader('DAV', '1,2', 1)
+
+        # Microsoft Web Folders compatibility, only enabled if
+        # User-Agent matches.
+        if ms_dav_agent.match(REQUEST.get_header('User-Agent', '')):
+            if webdav.enable_ms_public_header:
+                RESPONSE.setHeader('Public', ', '.join(self.__http_methods__))
+            if webdav.enable_ms_author_via:
+                RESPONSE.setHeader('MS-Author-Via', 'DAV')
+
         RESPONSE.setStatus(200)
         return RESPONSE
 
-    TRACE__roles__=None
+    security.declarePublic('TRACE')
     def TRACE(self, REQUEST, RESPONSE):
         """Return the HTTP message received back to the client as the
         entity-body of a 200 (OK) response. This will often usually
@@ -222,6 +239,7 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
         self.dav__init(REQUEST, RESPONSE)
         raise MethodNotAllowed, 'Method not supported for this resource.'
 
+    security.declareProtected(delete_objects, 'DELETE')
     def DELETE(self, REQUEST, RESPONSE):
         """Delete a resource. For non-collection resources, DELETE may
         return either 200 or 204 (No Content) to indicate success."""
@@ -229,7 +247,7 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
         ifhdr = REQUEST.get_header('If', '')
         url = urlfix(REQUEST['URL'], 'DELETE')
         name = unquote(filter(None, url.split( '/')[-1]))
-        parent = self.aq_parent
+        parent = aq_parent(aq_inner(self))
         # Lock checking
         if Lockable.wl_isLocked(self):
             if ifhdr:
@@ -260,6 +278,7 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
 
         return RESPONSE
 
+    security.declareProtected(webdav_access, 'PROPFIND')
     def PROPFIND(self, REQUEST, RESPONSE):
         """Retrieve properties defined on the resource."""
         self.dav__init(REQUEST, RESPONSE)
@@ -277,6 +296,7 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
         RESPONSE.setBody(result)
         return RESPONSE
 
+    security.declareProtected(manage_properties, 'PROPPATCH')
     def PROPPATCH(self, REQUEST, RESPONSE):
         """Set and/or remove properties defined on the resource."""
         self.dav__init(REQUEST, RESPONSE)
@@ -304,7 +324,7 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
         self.dav__init(REQUEST, RESPONSE)
         raise MethodNotAllowed, 'The resource already exists.'
 
-    COPY__roles__=('Anonymous',)
+    security.declarePublic('COPY')
     def COPY(self, REQUEST, RESPONSE):
         """Create a duplicate of the source resource whose state
         and behavior match that of the source resource as closely
@@ -393,10 +413,14 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
         if depth=='0' and isDavCollection(ob):
             for id in ob.objectIds():
                 ob._delObject(id)
+
+        notify(ObjectCopiedEvent(ob, self))
+
         if existing:
             object=getattr(parent, name)
             self.dav__validate(object, 'DELETE', REQUEST)
             parent._delObject(name)
+
         parent._setObject(name, ob)
         ob = parent._getOb(name)
         ob._postCopy(parent, op=0)
@@ -414,7 +438,7 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
         RESPONSE.setBody('')
         return RESPONSE
 
-    MOVE__roles__=('Anonymous',)
+    security.declarePublic('MOVE')
     def MOVE(self, REQUEST, RESPONSE):
         """Move a resource to a new location. Though we may later try to
         make a move appear seamless across namespaces (e.g. from Zope
@@ -502,20 +526,52 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
                 raise PreconditionFailed, 'Source is locked and no '\
                       'condition was passed in.'
 
+        orig_container = aq_parent(aq_inner(self))
+        orig_id = self.getId()
+
+        self._notifyOfCopyTo(parent, op=1)
+
+        notify(ObjectWillBeMovedEvent(self, orig_container, orig_id,
+                                      parent, name))
+
         # try to make ownership explicit so that it gets carried
         # along to the new location if needed.
         self.manage_changeOwnershipType(explicit=1)
 
-        self._notifyOfCopyTo(parent, op=1)
-        ob = aq_base(self._getCopy(parent))
-        self.aq_parent._delObject(absattr(self.id))
+        ob = self._getCopy(parent)
         ob._setId(name)
+
+        try:
+            orig_container._delObject(orig_id, suppress_events=True)
+        except TypeError:
+            # BBB: removed in Zope 2.11
+            orig_container._delObject(orig_id)
+            warnings.warn(
+                "%s._delObject without suppress_events is deprecated "
+                "and will be removed in Zope 2.11." %
+                orig_container.__class__.__name__, DeprecationWarning)
+
         if existing:
             object=getattr(parent, name)
             self.dav__validate(object, 'DELETE', REQUEST)
             parent._delObject(name)
-        parent._setObject(name, ob)
+
+        try:
+            parent._setObject(name, ob, set_owner=0, suppress_events=True)
+        except TypeError:
+            # BBB: removed in Zope 2.11
+            parent._setObject(name, ob, set_owner=0)
+            warnings.warn(
+                "%s._setObject without suppress_events is deprecated "
+                "and will be removed in Zope 2.11." %
+                parent.__class__.__name__, DeprecationWarning)
         ob = parent._getOb(name)
+
+        notify(ObjectMovedEvent(ob, orig_container, orig_id, parent, name))
+        notifyContainerModified(orig_container)
+        if aq_base(orig_container) is not aq_base(parent):
+            notifyContainerModified(parent)
+
         ob._postCopy(parent, op=1)
 
         # try to make ownership implicit if possible
@@ -530,6 +586,7 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
 
     # WebDAV Class 2, Lock and Unlock
 
+    security.declareProtected(webdav_lock_items, 'LOCK')
     def LOCK(self, REQUEST, RESPONSE):
         """Lock a resource"""
         self.dav__init(REQUEST, RESPONSE)
@@ -589,6 +646,7 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
 
         return RESPONSE
 
+    security.declareProtected(webdav_unlock_items, 'UNLOCK')
     def UNLOCK(self, REQUEST, RESPONSE):
         """Remove an existing lock on a resource."""
         self.dav__init(REQUEST, RESPONSE)
@@ -609,12 +667,14 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
         return RESPONSE
 
 
+    security.declareProtected(webdav_access, 'manage_DAVget')
     def manage_DAVget(self):
         """Gets the document source"""
         # The default implementation calls manage_FTPget
         return self.manage_FTPget()
 
+    security.declareProtected(webdav_access, 'listDAVObjects')
     def listDAVObjects(self):
         return []
 
-Globals.default__class_init__(Resource)
+InitializeClass(Resource)
