@@ -13,6 +13,15 @@
 ##############################################################################
 """Event Package
 
-$Id: __init__.py 27082 2004-08-12 20:03:58Z srichter $
+$Id: __init__.py 67630 2006-04-27 00:54:03Z jim $
 """
-from dispatching import publish
+__docformat__ = 'restructuredtext'
+
+import zope.deferredimport
+zope.deferredimport.deprecated(
+    "Event support has been moved to zope.component.event, life cycle events "
+    "to zope.lifecycleevent.  This reference will be gone in Zope 3.5",
+    objectevent = 'zope.lifecycleevent',
+    interfaces = 'zope.lifecycleevent.interfaces',
+    dispatch = 'zope.component.event',
+    )
