@@ -11,19 +11,30 @@
 ##############################################################################
 """General exceptions
 
-$Id: interfaces.py 25177 2004-06-02 13:17:31Z jim $
+$Id: interfaces.py 67630 2006-04-27 00:54:03Z jim $
 """
+__docformat__ = 'restructuredtext'
 
 from zope.interface import Interface, implements
 
-class IUserError(Interface):
-    """User error exceptions
+##############################################################################
+# BBB 2006/04/03 - to be removed after 12 months
+
+import zope.deferredimport
+zope.deferredimport.deprecated(
+    "UserError has been moved to zope.exceptions.interfaces. This "
+    "reference will be removed in Zope 3.5.",
+    UserError = 'zope.exceptions.interfaces:UserError',
+    IUserError = 'zope.exceptions.interfaces:IUserError',
+    )
+
+#
+##############################################################################
+
+class ISystemErrorView(Interface):
+    """Error views that can classify their contexts as system errors
     """
 
-class UserError(Exception):
-    """User errors
-
-    These exceptions should generally be displayed to users unless
-    they are handled.
-    """
-    implements(IUserError)
+    def isSystemError():
+        """Return a boolean indicating whether the error is a system errror
+        """
