@@ -13,17 +13,16 @@
 ##############################################################################
 """Folder interfaces
 
-$Id: interfaces.py 27516 2004-09-13 19:31:20Z fdrake $
+$Id: interfaces.py 67630 2006-04-27 00:54:03Z jim $
 """
 __docformat__ = 'restructuredtext'
 
-from zope.app.container.interfaces import IContainer, IContentContainer
-from zope.app.traversing.interfaces import IContainmentRoot
-from zope.app.site.interfaces import IPossibleSite
-from zope.app.annotation.interfaces import IAttributeAnnotatable
+from zope.traversing.interfaces import IContainmentRoot
+from zope.annotation.interfaces import IAttributeAnnotatable
+from zope.app.container.interfaces import IContainer
+from zope.app.component.interfaces import IPossibleSite
 
-class IFolder(IContainer, IContentContainer, IPossibleSite,
-              IAttributeAnnotatable):
+class IFolder(IContainer, IPossibleSite, IAttributeAnnotatable):
     """The standard Zope Folder object interface."""
 
 class IRootFolder(IFolder, IContainmentRoot):
