@@ -13,7 +13,7 @@
 ##############################################################################
 """Generic Widget base classes
 
-$Id: __init__.py 70363 2006-09-25 12:34:15Z yuppie $
+$Id: __init__.py 71639 2006-12-20 23:55:22Z jacobholm $
 """
 __docformat__ = 'restructuredtext'
 
@@ -60,7 +60,7 @@ class Widget(object):
         return self._data is not self._data_marker
 
     def setPrefix(self, prefix):
-        if not prefix.endswith("."):
+        if prefix and not prefix.endswith("."):
             prefix += '.'
         self._prefix = prefix
         self.name = prefix + self.context.__name__
