@@ -13,30 +13,27 @@
 ##############################################################################
 """Form Utilities Tests
 
-$Id: test_utility.py 27079 2004-08-12 19:45:55Z srichter $
+$Id: test_utility.py 69217 2006-07-20 03:56:26Z baijum $
 """
-import doctest
 
+from zope.testing import doctest
+import zope.security.checker
 from zope.interface import Interface, implements
-from zope.component.interfaces import IViewFactory
-from zope.component.exceptions import ComponentLookupError
-from zope.publisher.browser import TestRequest
+from zope.component.interfaces import IViewFactory, ComponentLookupError
+from zope.publisher.browser import TestRequest, BrowserView
+from zope.security.interfaces import ForbiddenAttribute, Unauthorized
+from zope.schema import Field, Int, accessors
+from zope.schema.interfaces import IField, IInt
 
-from zope.schema import Field
-from zope.schema.interfaces import IField
-
-from zope.app.tests import ztapi
-from zope.app.publisher.browser import BrowserView
+from zope.app.testing import ztapi, placelesssetup
 from zope.app.form import Widget
-from zope.app.form.interfaces import IWidget, IInputWidget
-from zope.app.form.interfaces import IDisplayWidget, WidgetsError
-
+from zope.app.form.interfaces import IWidget, IInputWidget, IDisplayWidget
+from zope.app.form.interfaces import ConversionError, InputErrors, WidgetsError
 from zope.app.form.utility import no_value, setUpWidget, setUpWidgets
 from zope.app.form.utility import setUpEditWidgets, setUpDisplayWidgets
 from zope.app.form.utility import getWidgetsData, viewHasInput
 from zope.app.form.utility import applyWidgetsChanges
-
-from zope.app.tests import placelesssetup
+from zope.app.form.tests import utils
 
 request = TestRequest()
 
@@ -51,18 +48,26 @@ class IBar(IField):
     
 class Bar(Field):
     implements(IBar)
+
+class IBaz(IInt):
+    pass
+
+class Baz(Int):
+    implements(IBaz)
     
 class IContent(Interface):
     foo = Foo()
     bar = Bar()
+
     
 class Content(object):
     implements(IContent)
+    __Security_checker__ = utils.SchemaChecker(IContent)
     foo = 'Foo'
-    
+
 class IFooWidget(IWidget):
     pass
-    
+
 class IBarWidget(IWidget):
     pass
 
@@ -76,7 +81,29 @@ class BarWidget(Widget):
     implements(IBarWidget)
     def getRenderedValue(self): return self._data # exposes _data for testing
     def renderedValueSet(self): return self._renderedValueSet() # for testing
-        
+
+class BazWidget(Widget):
+    def getRenderedValue(self): return self._data # exposes _data for testing
+    def renderedValueSet(self): return self._renderedValueSet() # for testing
+
+class IExtendedContent(IContent):
+    getBaz, setBaz = accessors(Baz())
+    getAnotherBaz, setAnotherBaz = accessors(Baz())
+    shazam = Foo()
+
+class ExtendedContent(Content):
+    implements(IExtendedContent)
+    _baz = _anotherbaz = shazam = None
+    def getBaz(self): return self._baz
+    def setBaz(self, value): self._baz = value
+    def getAnotherBaz(self): return self._anotherbaz
+    def setAnotherBaz(self, value): self._anotherbaz = value
+
+extended_checker = utils.DummyChecker(
+        {'foo':True, 'bar': True, 'getBaz': True, 'setBaz': True, 
+         'getAnotherBaz': True, 'setAnotherBaz': False, 'shazam': False},
+        {'foo':True, 'bar': False, 'shazam': True})
+
 def setUp():
     """Setup for tests."""
     placelesssetup.setUp()
@@ -505,7 +532,7 @@ class TestSetUpWidgets(object):
             >>> setUpWidgets(view, IContent, IWidget, 'prefix', True,
             ...              initial={ "bar":"Bar" },
             ...              context="Alt Context")
-            view: <class 'zope.app.publisher.browser.BrowserView'>
+            view: <class 'zope.publisher.browser.BrowserView'>
             name: foo
             field: <class 'zope.app.form.tests.test_utility.Foo'>
             viewType: <class 'zope.interface.interface.InterfaceClass'>
@@ -514,7 +541,7 @@ class TestSetUpWidgets(object):
             ignoreStickyValues: True
             context: Alt Context
             ---
-            view: <class 'zope.app.publisher.browser.BrowserView'>
+            view: <class 'zope.publisher.browser.BrowserView'>
             name: bar
             field: <class 'zope.app.form.tests.test_utility.Bar'>
             viewType: <class 'zope.interface.interface.InterfaceClass'>
@@ -527,6 +554,44 @@ class TestSetUpWidgets(object):
      
         >>> tearDown()
         """
+
+    def test_forbiddenAttributes(self):
+        """Tests that forbidden attributes cause an error in widget setup.
+
+        >>> setUp()
+
+        If an attribute cannot be read from a source object because it's
+        forbidden, the ForbiddenAttribute error is allowed to pass through
+        to the caller.
+
+        We'll create a field that raises a ForbiddenError itself to simulate
+        what would happen when a proxied object's attribute is accessed without
+        the required permission.
+
+            >>> class AlwaysForbidden(Field):
+            ...     def get(self, source):
+            ...         raise ForbiddenAttribute(source, self.__name__)
+
+        We'll also create a schema using this field:
+
+            >>> class IMySchema(Interface):
+            ...     tryme = AlwaysForbidden()
+
+        When we use setUpEditWidgets to configure a view with IMySchema:
+
+            >>> view = BrowserView('some context', TestRequest())
+            >>> setUpEditWidgets(view, IMySchema)
+            Traceback (most recent call last):
+            ForbiddenAttribute: ('some context', 'tryme')
+
+        The same applies to setUpDisplayWidgets:
+
+            >>> setUpDisplayWidgets(view, IMySchema)
+            Traceback (most recent call last):
+            ForbiddenAttribute: ('some context', 'tryme')
+
+        >>> tearDown()
+        """
         
 class TestFormSetUp(object):
     
@@ -560,6 +625,7 @@ class TestFormSetUp(object):
         A call to setUpEditWidgets with the view:
             
             >>> setUpEditWidgets(view, IContent)
+            ['foo', 'bar']
             
         configures the view with widgets that accept input for the context 
         field values:
@@ -581,6 +647,7 @@ class TestFormSetUp(object):
             >>> source.foo = 'abc2'
             >>> source.bar = 'def2'
             >>> setUpEditWidgets(view, IContent, source=source)
+            ['foo', 'bar']
             >>> view.foo_widget.getRenderedValue()
             'abc2'
             >>> view.bar_widget.getRenderedValue()
@@ -596,10 +663,115 @@ class TestFormSetUp(object):
             >>> IContent['foo'].readonly = True
             >>> delattr(view, 'foo_widget')
             >>> setUpEditWidgets(view, IContent)
+            ['foo', 'bar']
             >>> isinstance(view.foo_widget, DisplayWidget)
             True
             >>> IContent['foo'].readonly = save  # restore readonly value
         
+        By default, setUpEditWidgets raises Unauthorized if it is asked to
+        set up a field to which the user does not have permission to
+        access or to change.  In the definition of the ExtendedContent
+        interface, notice the __Security_checker__ attribute, which stubs
+        out a checker that allows the user to view the bar attribute,
+        but not set it, and call getAnotherBaz but not setAnotherBaz.
+        
+            >>> view.context = context = zope.security.checker.Proxy(
+            ...     ExtendedContent(), extended_checker)
+            >>> setUpEditWidgets(view, IExtendedContent, names=['bar'])
+            ... # can' write to bar
+            Traceback (most recent call last):
+            ...
+            Unauthorized: bar
+            >>> setUpEditWidgets(
+            ... view, IExtendedContent, names=['getAnotherBaz'])
+            ... # can't access the setter, setAnotherBaz
+            Traceback (most recent call last):
+            ...
+            Unauthorized: setAnotherBaz
+            >>> setUpEditWidgets(
+            ... view, IExtendedContent, names=['shazam'])
+            ... # can't even access shazam
+            Traceback (most recent call last):
+            ...
+            Unauthorized
+        
+        Two optional flags can change this behavior.  degradeDisplay=True 
+        causes the form machinery to skip fields silently that the user may
+        not access.  In this case, the return value of setUpEditWidgets--
+        a list of the field names set up--will be different that the names
+        provided to the function.
+        
+            >>> delattr(view, 'foo_widget')
+            >>> delattr(view, 'bar_widget')
+            >>> ztapi.browserViewProviding(IBaz, InputWidget, IInputWidget)
+            >>> setUpEditWidgets(
+            ...     view, IExtendedContent, names=['foo', 'shazam', 'getBaz'],
+            ...     degradeDisplay=True)
+            ['foo', 'getBaz']
+            >>> IInputWidget.providedBy(view.foo_widget)
+            True
+            >>> IInputWidget.providedBy(view.getBaz_widget)
+            True
+            >>> view.shazam_widget
+            Traceback (most recent call last):
+            ...
+            AttributeError: 'BrowserView' object has no attribute 'shazam_widget'
+        
+        Similarly, degradeInput=True causes the function to degrade to
+        display widgets for any fields that the current user cannot change,
+        but can see.
+        
+            >>> delattr(view, 'foo_widget')
+            >>> delattr(view, 'getBaz_widget')
+            >>> ztapi.browserViewProviding(IBar, DisplayWidget, IDisplayWidget)
+            >>> ztapi.browserViewProviding(IBaz, DisplayWidget, IDisplayWidget)
+            >>> setUpEditWidgets(
+            ...     view, IExtendedContent, 
+            ...     names=['foo', 'bar', 'getBaz', 'getAnotherBaz'],
+            ...     degradeInput=True)
+            ['foo', 'bar', 'getBaz', 'getAnotherBaz']
+            >>> IInputWidget.providedBy(view.foo_widget)
+            True
+            >>> IDisplayWidget.providedBy(view.bar_widget)
+            True
+            >>> IInputWidget.providedBy(view.getBaz_widget)
+            True
+            >>> IDisplayWidget.providedBy(view.getAnotherBaz_widget)
+            True
+        
+        Note that if the user cannot view the current value then they cannot
+        view the input widget.  The two flags can then, of course, be used 
+        together.
+        
+            >>> delattr(view, 'foo_widget')
+            >>> delattr(view, 'bar_widget')
+            >>> delattr(view, 'getBaz_widget')
+            >>> delattr(view, 'getAnotherBaz_widget')
+            >>> setUpEditWidgets(
+            ...     view, IExtendedContent, 
+            ...     names=['foo', 'bar', 'shazam', 'getBaz', 'getAnotherBaz'],
+            ...     degradeInput=True)
+            Traceback (most recent call last):
+            ...
+            Unauthorized
+            >>> setUpEditWidgets(
+            ...     view, IExtendedContent, 
+            ...     names=['foo', 'bar', 'shazam', 'getBaz', 'getAnotherBaz'],
+            ...     degradeInput=True, degradeDisplay=True)
+            ['foo', 'bar', 'getBaz', 'getAnotherBaz']
+            >>> IInputWidget.providedBy(view.foo_widget)
+            True
+            >>> IDisplayWidget.providedBy(view.bar_widget)
+            True
+            >>> IInputWidget.providedBy(view.getBaz_widget)
+            True
+            >>> IDisplayWidget.providedBy(view.getAnotherBaz_widget)
+            True
+            >>> view.shazam_widget
+            Traceback (most recent call last):
+            ...
+            AttributeError: 'BrowserView' object has no attribute 'shazam_widget'
+        
         >>> tearDown()
         """
         
@@ -612,7 +784,7 @@ class TestFormSetUp(object):
         The function looks up widgets of type IDisplayWidget for the specified
         schema.
         
-        We'll first create and register widgets for the schame fields
+        We'll first create and register widgets for the schema fields
         we want to edit:
             
             >>> class DisplayWidget(Widget):
@@ -628,9 +800,10 @@ class TestFormSetUp(object):
             >>> context.bar = 'def'
             >>> view = BrowserView(context, request)
             
-        A call to setUpEditWidgets with the view:
+        A call to setUpDisplayWidgets with the view:
             
             >>> setUpDisplayWidgets(view, IContent)
+            ['foo', 'bar']
             
         configures the view with widgets that display the context fields:
             
@@ -651,11 +824,44 @@ class TestFormSetUp(object):
             >>> source.foo = 'abc2'
             >>> source.bar = 'def2'
             >>> setUpDisplayWidgets(view, IContent, source=source)
+            ['foo', 'bar']
             >>> view.foo_widget.getRenderedValue()
             'abc2'
             >>> view.bar_widget.getRenderedValue()
             'def2'
         
+        Also like setUpEditWidgets, the degradeDisplay flag allows widgets
+        to silently disappear if they are unavailable.
+        
+            >>> view.context = context = zope.security.checker.Proxy(
+            ...     ExtendedContent(), extended_checker)
+            >>> delattr(view, 'foo_widget')
+            >>> delattr(view, 'bar_widget')
+            >>> ztapi.browserViewProviding(IBaz, DisplayWidget, IDisplayWidget)
+            >>> setUpDisplayWidgets(
+            ...     view, IExtendedContent, 
+            ...     names=['foo', 'bar', 'shazam', 'getBaz', 'getAnotherBaz'])
+            Traceback (most recent call last):
+            ...
+            Unauthorized
+            >>> setUpDisplayWidgets(
+            ...     view, IExtendedContent, 
+            ...     names=['foo', 'bar', 'shazam', 'getBaz', 'getAnotherBaz'],
+            ...     degradeDisplay=True)
+            ['foo', 'bar', 'getBaz', 'getAnotherBaz']
+            >>> IDisplayWidget.providedBy(view.foo_widget)
+            True
+            >>> IDisplayWidget.providedBy(view.bar_widget)
+            True
+            >>> IDisplayWidget.providedBy(view.getBaz_widget)
+            True
+            >>> IDisplayWidget.providedBy(view.getAnotherBaz_widget)
+            True
+            >>> view.shazam_widget
+            Traceback (most recent call last):
+            ...
+            AttributeError: 'BrowserView' object has no attribute 'shazam_widget'
+        
         >>> tearDown()
         """
         
@@ -681,6 +887,7 @@ class TestForms(object):
             >>> ztapi.browserViewProviding(IBar, InputWidget, IInputWidget)
             >>> view = BrowserView(Content(), request)
             >>> setUpEditWidgets(view, IContent)
+            ['foo', 'bar']
             
         Because InputWidget is configured to not have input by default, the
         view does not have input:
@@ -712,20 +919,26 @@ class TestForms(object):
             >>> class InputWidget(Widget):
             ...     implements(IInputWidget)           
             ...     input = None
+            ...     valid = True
             ...     def hasInput(self):
             ...         return input is not None
             ...     def applyChanges(self, object):
+            ...         if not self.valid:
+            ...             raise ConversionError('invalid input')
             ...         field = self.context
             ...         field.set(object, self.input)
             ...         return True
             >>> ztapi.browserViewProviding(IFoo, InputWidget, IInputWidget)
+            >>> ztapi.browserViewProviding(IBar, InputWidget, IInputWidget)
             
         Before calling applyWidgetsUpdate, we need to configure a context and
         a view with edit widgets:
             
             >>> context = Content()
             >>> view = BrowserView(context, request)
-            >>> setUpEditWidgets(view, IContent, names=('foo',))
+            >>> setUpEditWidgets(
+            ...     view, IContent, context=context, names=('foo',))
+            ['foo']
             
         We now specify new widget input and apply the changes: 
 
@@ -761,6 +974,32 @@ class TestForms(object):
             Traceback (most recent call last):
             AttributeError: 'BrowserView' object has no attribute 'foo_widget'
 
+        When applyWidgetsChanges is called with multiple form
+        fields, some with valid data and some with invalid data, 
+        *changes may be applied*.  For instance, below see that context.foo
+        changes from 'Foo' to 'a' even though trying to change context.bar
+        fails.  Generally, ZODB transactional behavior is expected to
+        correct this sort of problem.
+
+            >>> context = Content()
+            >>> view = BrowserView(context, request)
+            >>> setUpEditWidgets(view, IContent, names=('foo', 'bar'))
+            ['foo', 'bar']
+            >>> view.foo_widget.input = 'a'
+            >>> view.bar_widget.input = 'b'
+            >>> view.bar_widget.valid = False
+            >>> context.foo
+            'Foo'
+            >>> getattr(context, 'bar', 'not really')
+            'not really'
+            >>> applyWidgetsChanges(view, IContent, names=('foo', 'bar'))
+            Traceback (most recent call last):
+            WidgetsError: ConversionError: ('invalid input', None)
+            >>> context.foo
+            'a'
+            >>> getattr(context, 'bar', 'not really')
+            'not really'
+
         >>> tearDown()
         """
         
@@ -790,6 +1029,7 @@ class TestGetWidgetsData(object):
         
             >>> view = BrowserView(Content(), request)
             >>> setUpEditWidgets(view, IContent)
+            ['foo', 'bar']
             
         The simplest form of getWidgetsData requires a view and a schema:
             
