@@ -13,11 +13,11 @@
 ##############################################################################
 """Object Interface Vocabulary Tests
 
-$Id: test_vocabulary.py 27506 2004-09-12 14:51:13Z philikon $
+$Id: test_vocabulary.py 29143 2005-02-14 22:43:16Z srichter $
 """
 import unittest
 from zope.testing.doctestunit import DocTestSuite
-from zope.app.tests import setup
+from zope.app.testing import setup
 
 def test_suite():
     return DocTestSuite('zope.app.interface.vocabulary',
