@@ -14,13 +14,15 @@
 ##############################################################################
 """HTTP Publisher Tests
 
-$Id: test_http.py 29588 2005-03-20 00:19:47Z BjornT $
+$Id: test_http.py 69340 2006-08-02 14:16:04Z mj $
 """
 import unittest
 
+import zope.event
 from zope.interface import implements
 from zope.publisher.interfaces.logginginfo import ILoggingInfo
 from zope.publisher.http import HTTPRequest, HTTPResponse
+from zope.publisher.http import HTTPInputStream, StrResult
 from zope.publisher.publish import publish
 from zope.publisher.base import DefaultPublication
 from zope.publisher.interfaces.http import IHTTPRequest, IHTTPResponse
@@ -32,7 +34,7 @@ from zope.i18n.interfaces.locales import ILocale
 from zope.interface.verify import verifyObject
 
 from StringIO import StringIO
-from Cookie import SimpleCookie, CookieError
+from Cookie import CookieError
 
 
 class UserStub(object):
@@ -48,6 +50,60 @@ class UserStub(object):
         return self._id
 
 
+data = '''\
+line 1
+line 2
+line 3'''
+
+
+class HTTPInputStreamTests(unittest.TestCase):
+
+    def setUp(self):
+        self.stream = HTTPInputStream(StringIO(data), {})
+
+    def getCacheStreamValue(self):
+        self.stream.cacheStream.seek(0)
+        result = self.stream.cacheStream.read()
+        # We just did a read on a file opened for update.  If the next
+        # operation on that file is a write, behavior is 100% undefined,
+        # and it in fact frequently (but not always) blows up on Windows.
+        # Behavior is 100% defined instead if we explictly seek.  Since
+        # we expect to be at EOF now, explicitly seek to the end.
+        self.stream.cacheStream.seek(0, 2)
+        return result
+
+    def testRead(self):
+        output = ''
+        self.assertEqual(output, self.getCacheStreamValue())
+        output += self.stream.read(5)
+        self.assertEqual(output, self.getCacheStreamValue())
+        output += self.stream.read()
+        self.assertEqual(output, self.getCacheStreamValue())
+        self.assertEqual(data, self.getCacheStreamValue())
+
+    def testReadLine(self):
+        output = self.stream.readline()
+        self.assertEqual(output, self.getCacheStreamValue())
+        output += self.stream.readline()
+        self.assertEqual(output, self.getCacheStreamValue())
+        output += self.stream.readline()
+        self.assertEqual(output, self.getCacheStreamValue())
+        output += self.stream.readline()
+        self.assertEqual(output, self.getCacheStreamValue())
+        self.assertEqual(data, self.getCacheStreamValue())
+
+    def testReadLines(self):
+        output = ''.join(self.stream.readlines(4))
+        self.assertEqual(output, self.getCacheStreamValue())
+        output += ''.join(self.stream.readlines())
+        self.assertEqual(output, self.getCacheStreamValue())
+        self.assertEqual(data, self.getCacheStreamValue())
+
+    def testGetChacheStream(self):
+        self.stream.read(5)
+        self.assertEqual(data, self.stream.getCacheStream().read())
+
+
 class HTTPTests(unittest.TestCase):
 
     _testEnv =  {
@@ -71,7 +127,7 @@ class HTTPTests(unittest.TestCase):
             """Required docstring for the publisher."""
 
         class Item(object):
-            """Required docstring for the publisher."""            
+            """Required docstring for the publisher."""
             def __call__(self, a, b):
                 return "%s, %s" % (`a`, `b`)
 
@@ -80,26 +136,31 @@ class HTTPTests(unittest.TestCase):
         self.app.folder.item = Item()
         self.app.xxx = Item()
 
-    def _createRequest(self, extra_env={}, body="", outstream=None):
+    def _createRequest(self, extra_env={}, body=""):
         env = self._testEnv.copy()
         env.update(extra_env)
         if len(body):
             env['CONTENT_LENGTH'] = str(len(body))
 
         publication = DefaultPublication(self.app)
-        if outstream is None:
-            outstream = StringIO()
         instream = StringIO(body)
-        request = HTTPRequest(instream, outstream, env)
+        request = HTTPRequest(instream, env)
         request.setPublication(publication)
         return request
 
     def _publisherResults(self, extra_env={}, body=""):
-        outstream = StringIO()
-        request = self._createRequest(extra_env, body, outstream=outstream)
-        publish(request, handle_errors=0)
-        return outstream.getvalue()
-
+        request = self._createRequest(extra_env, body)
+        response = request.response
+        publish(request, handle_errors=False)
+        headers = response.getHeaders()
+        headers.sort()
+        return (
+            "Status: %s\r\n" % response.getStatusString()
+            +
+            "\r\n".join([("%s: %s" % h) for h in headers]) + "\r\n\r\n"
+            +
+            ''.join(response.consumeBody())
+            )
 
     def test_repr(self):
         request = self._createRequest()
@@ -125,7 +186,7 @@ class HTTPTests(unittest.TestCase):
         location = request.response.redirect('http://foobar.com/redirected')
         self.assertEquals(location, 'http://foobar.com/redirected')
         self.assertEquals(request.response.getStatus(), 302)
-        self.assertEquals(request.response._headers['location'], location)
+        self.assertEquals(request.response.getHeader('location'), location)
 
         # test HTTP/1.1
         env = {'SERVER_PROTOCOL':'HTTP/1.1'}
@@ -172,6 +233,14 @@ class HTTPTests(unittest.TestCase):
     def testRequestLocale(self):
         eq = self.assertEqual
         unless = self.failUnless
+
+        from zope.component import provideAdapter
+        from zope.publisher.browser import BrowserLanguages
+        from zope.publisher.interfaces.http import IHTTPRequest
+        from zope.i18n.interfaces import IUserPreferredLanguages
+        provideAdapter(BrowserLanguages, [IHTTPRequest],
+                       IUserPreferredLanguages)
+
         for httplang in ('it', 'it-ch', 'it-CH', 'IT', 'IT-CH', 'IT-ch'):
             req = self._createRequest({'HTTP_ACCEPT_LANGUAGE': httplang})
             locale = req.locale
@@ -211,6 +280,9 @@ class HTTPTests(unittest.TestCase):
         eq(locale.id.territory, None)
         eq(locale.id.variant, None)
 
+        from zope.component.testing import tearDown
+        tearDown()
+
     def testCookies(self):
         cookies = {
             'HTTP_COOKIE':
@@ -230,6 +302,25 @@ class HTTPTests(unittest.TestCase):
         # Reserved key
         self.failIf(req.cookies.has_key('path'))
 
+    def testCookieErrorToLog(self):
+        cookies = {
+            'HTTP_COOKIE':
+                'foo=bar; path=/; spam="eggs", ldap/OU="Williams"'
+        }
+        req = self._createRequest(extra_env=cookies)
+
+        self.failIf(req.cookies.has_key('foo'))
+        self.failIf(req.has_key('foo'))
+
+        self.failIf(req.cookies.has_key('spam'))
+        self.failIf(req.has_key('spam'))
+
+        self.failIf(req.cookies.has_key('ldap/OU'))
+        self.failIf(req.has_key('ldap/OU'))
+
+        # Reserved key
+        self.failIf(req.cookies.has_key('path'))
+
     def testCookiesUnicode(self):
         # Cookie values are assumed to be UTF-8 encoded
         cookies = {'HTTP_COOKIE': r'key="\342\230\243";'}
@@ -253,41 +344,22 @@ class HTTPTests(unittest.TestCase):
 
     def testBasicAuth(self):
         from zope.publisher.interfaces.http import IHTTPCredentials
-        import base64
         req = self._createRequest()
         verifyObject(IHTTPCredentials, req)
         lpq = req._authUserPW()
         self.assertEquals(lpq, None)
         env = {}
-        login, password = ("tim", "123")
-        s = base64.encodestring("%s:%s" % (login, password)).rstrip()
+        login, password = ("tim", "123:456")
+        s = ("%s:%s" % (login, password)).encode("base64").rstrip()
         env['HTTP_AUTHORIZATION'] = "Basic %s" % s
         req = self._createRequest(env)
         lpw = req._authUserPW()
         self.assertEquals(lpw, (login, password))
 
     def testSetPrincipal(self):
-        class HTTPTaskStub(object):
-            auth_user_name = None
-            def setAuthUserName(self, name):
-                self.auth_user_name = name
-
-        task = HTTPTaskStub()
-        req = self._createRequest(outstream=task)
+        req = self._createRequest()
         req.setPrincipal(UserStub("jim"))
-        self.assert_(not req.response._outstream.auth_user_name)
-        req = self._createRequest(outstream=task)
-        req.response.setHTTPTransaction(task)
-        req.setPrincipal(UserStub("jim"))
-        self.assertEquals(req.response.http_transaction.auth_user_name, "jim")
-
-    def testIPresentationRequest(self):
-        # test the IView request
-        r = self._createRequest()
-
-        self.assertEqual(r.getPresentationSkin(), '')
-        r.setPresentationSkin('morefoo')
-        self.assertEqual(r.getPresentationSkin(), 'morefoo')
+        self.assertEquals(req.response.authUser, 'jim')
 
     def test_method(self):
         r = self._createRequest(extra_env={'REQUEST_METHOD':'SPAM'})
@@ -296,6 +368,8 @@ class HTTPTests(unittest.TestCase):
         self.assertEqual(r.method, 'EGGS')
 
     def test_setApplicationServer(self):
+        events = []
+        zope.event.subscribers.append(events.append)
         req = self._createRequest()
         req.setApplicationServer('foo')
         self.assertEquals(req._app_server, 'http://foo')
@@ -313,22 +387,36 @@ class HTTPTests(unittest.TestCase):
         self.assertEquals(req._app_server, 'http://foo')
         req.setApplicationServer('foo', proto='telnet', port=80)
         self.assertEquals(req._app_server, 'telnet://foo:80')
+        zope.event.subscribers.pop()
+        self.assertEquals(len(events), 8)
+        for event in events:
+            self.assertEquals(event.request, req)
 
     def test_setApplicationNames(self):
+        events = []
+        zope.event.subscribers.append(events.append)
         req = self._createRequest()
         names = ['x', 'y', 'z']
         req.setVirtualHostRoot(names)
         self.assertEquals(req._app_names, ['x', 'y', 'z'])
         names[0] = 'muahahahaha'
         self.assertEquals(req._app_names, ['x', 'y', 'z'])
+        zope.event.subscribers.pop()
+        self.assertEquals(len(events), 1)
+        self.assertEquals(events[0].request, req)
 
     def test_setVirtualHostRoot(self):
+        events = []
+        zope.event.subscribers.append(events.append)
         req = self._createRequest()
         req._traversed_names = ['x', 'y']
         req._last_obj_traversed = object()
         req.setVirtualHostRoot()
         self.failIf(req._traversed_names)
         self.assertEquals(req._vh_root, req._last_obj_traversed)
+        zope.event.subscribers.pop()
+        self.assertEquals(len(events), 1)
+        self.assertEquals(events[0].request, req)
 
     def test_getVirtualHostRoot(self):
         req = self._createRequest()
@@ -388,6 +476,11 @@ class HTTPTests(unittest.TestCase):
         # the request should have converted PATH_INFO to unicode
         self.assertEqual(req['PATH_INFO'], u'///foo/bar.html')
 
+    def testResponseWriteFaile(self):
+        self.assertRaises(TypeError,
+                          self._createRequest().response.write,
+                          'some output',
+                          )
 
 class ConcreteHTTPTests(HTTPTests):
     """Tests that we don't have to worry about subclasses inheriting and
@@ -417,75 +510,40 @@ class ConcreteHTTPTests(HTTPTests):
 class TestHTTPResponse(unittest.TestCase):
 
     def testInterface(self):
-        rp = HTTPResponse(StringIO())
+        rp = HTTPResponse()
         verifyObject(IHTTPResponse, rp)
         verifyObject(IHTTPApplicationResponse, rp)
         verifyObject(IResponse, rp)
 
     def _createResponse(self):
-        stream = StringIO()
-        response = HTTPResponse(stream)
-        return response, stream
+        response = HTTPResponse()
+        return response
 
-    def _parseResult(self, result):
-        hdrs_text, body = result.split("\r\n\r\n", 1)
-        headers = {}
-        for line in hdrs_text.splitlines():
-            key, val = line.split(":", 1)
-            key = key.strip()
-            val = val.strip()
-            if headers.has_key(key):
-                if type(headers[key]) == type([]):
-                    headers[key].append(val)
-                else:
-                    headers[key] = [headers[key], val]
-            else:
-                headers[key] = val
-        return headers, body
+    def _parseResult(self, response):
+        return dict(response.getHeaders()), ''.join(response.consumeBody())
 
-    def _getResultFromResponse(self, body, charset=None, headers=None):
-        response, stream = self._createResponse()
-        if charset is not None:
-            response.setCharset(charset)
+    def _getResultFromResponse(self, body, charset='utf-8', headers=None):
+        response = self._createResponse()
+        assert(charset == 'utf-8')
         if headers is not None:
             for hdr, val in headers.iteritems():
                 response.setHeader(hdr, val)
-        response.setBody(body)
-        response.outputBody()
-        return self._parseResult(stream.getvalue())
-
-    def testWrite(self):
-        response, stream = self._createResponse()
-        data = 'a'*10
-        # We have to set all the headers ourself
-        response.setHeader('Content-Type', 'text/plain;charset=us-ascii')
-        response.setHeader('Content-Length', str(len(data)))
-
-        # Stream the data
-        for ch in data:
-            response.write(ch)
-
-        headers, body = self._parseResult(stream.getvalue())
-        # Check that the data have been written, and that the header
-        # has been preserved   
-        self.assertEqual(headers['Content-Type'], 'text/plain;charset=us-ascii')
-        self.assertEqual(headers['Content-Length'], str(len(data)))
-        self.assertEqual(body, data)
+        response.setResult(body)
+        return self._parseResult(response)
 
     def testWrite_noContentLength(self):
-        response, stream = self._createResponse()
-        data = 'a'*10
+        response = self._createResponse()
         # We have to set all the headers ourself, we choose not to provide a
         # content-length header
         response.setHeader('Content-Type', 'text/plain;charset=us-ascii')
 
-        # Stream the data
-        for ch in data:
-            response.write(ch)
+        # Output the data
+        data = 'a'*10
+        response.setResult(StrResult(data))
 
-        headers, body = self._parseResult(stream.getvalue())
+        headers, body = self._parseResult(response)
         # Check that the data have been written, and that the header
-        # has been preserved   
+        # has been preserved
         self.assertEqual(headers['Content-Type'], 'text/plain;charset=us-ascii')
         self.assertEqual(body, data)
 
@@ -513,17 +571,17 @@ class TestHTTPResponse(unittest.TestCase):
         eq("", headers.get("Content-Type", ""))
         eq("test", body)
 
-        headers, body = self._getResultFromResponse("test",
+        headers, body = self._getResultFromResponse(u"test",
             headers={"content-type": "text/plain"})
         eq("text/plain;charset=utf-8", headers["Content-Type"])
         eq("test", body)
 
-        headers, body = self._getResultFromResponse("test", "utf-8",
+        headers, body = self._getResultFromResponse(u"test", "utf-8",
             {"content-type": "text/html"})
         eq("text/html;charset=utf-8", headers["Content-Type"])
         eq("test", body)
 
-        headers, body = self._getResultFromResponse("test", "utf-8",
+        headers, body = self._getResultFromResponse(u"test", "utf-8",
             {"content-type": "text/plain;charset=cp1251"})
         eq("text/plain;charset=cp1251", headers["Content-Type"])
         eq("test", body)
@@ -536,23 +594,19 @@ class TestHTTPResponse(unittest.TestCase):
     def _getCookieFromResponse(self, cookies):
         # Shove the cookies through request, parse the Set-Cookie header
         # and spit out a list of headers for examination
-        response, stream = self._createResponse()
+        response = self._createResponse()
         for name, value, kw in cookies:
             response.setCookie(name, value, **kw)
-        response.setBody('test')
-        response.outputBody()
-        headers, body = self._parseResult(stream.getvalue())
-        c = SimpleCookie()
-        cookie_headers = headers["Set-Cookie"]
-        if type(cookie_headers) != type([]):
-            cookie_headers = [cookie_headers]
-        return cookie_headers
+        response.setResult('test')
+        return [header[1]
+                for header in response.getHeaders()
+                if header[0] == "Set-Cookie"]
 
     def testSetCookie(self):
         c = self._getCookieFromResponse([
                 ('foo', 'bar', {}),
                 ])
-        self.failUnless('foo=bar;' in c, 'foo=bar not in %r' % c)
+        self.failUnless('foo=bar;' in c, 'foo=bar; not in %r' % c)
 
         c = self._getCookieFromResponse([
                 ('foo', 'bar', {}),
@@ -599,6 +653,7 @@ def test_suite():
     suite = unittest.TestSuite()
     suite.addTest(unittest.makeSuite(ConcreteHTTPTests))
     suite.addTest(unittest.makeSuite(TestHTTPResponse))
+    suite.addTest(unittest.makeSuite(HTTPInputStreamTests))
     return suite
 
 
